# -*- coding: utf-8 -*-
import collections
from abjad.tools import stringtools
from abjad.tools import systemtools
from abjad.tools.abctools import AbjadValueObject


class Scheme(AbjadValueObject):
    r'''Abjad model of Scheme code.

    ..  container:: example

        **Example 1.** A Scheme boolean value:

        ::

            >>> scheme = schemetools.Scheme(True)
            >>> print(format(scheme))
            ##t

    ..  container:: example

        **Example 2.** A nested Scheme expession:

        ::

            >>> scheme = schemetools.Scheme(
            ...     ('left', (1, 2, False)),
            ...     ('right', (1, 2, 3.3))
            ...     )
            >>> print(format(scheme))
            #((left (1 2 #f)) (right (1 2 3.3)))

    ..  container:: example

        **Example 3.** A variable-length argument:

        ::

            >>> scheme_1 = schemetools.Scheme(1, 2, 3)
            >>> scheme_2 = schemetools.Scheme((1, 2, 3))
            >>> format(scheme_1) == format(scheme_2)
            True

        Scheme wraps nested variable-length arguments in a tuple.

    ..  container:: example

        **Example 4.** A quoted Scheme expression:

        ::

            >>> scheme = schemetools.Scheme((1, 2, 3), quoting="'#")
            >>> print(format(scheme))
            #'#(1 2 3)

        Use the `quoting` keyword to prepend Scheme's various quote, unquote,
        unquote-splicing characters to formatted output.

    ..  container:: example

        **Example 5.** A Scheme expression with forced quotes:

        ::

            >>> scheme = schemetools.Scheme('nospaces', force_quotes=True)
            >>> print(format(scheme))
            #"nospaces"

        Use this in certain \override situations when LilyPond's Scheme
        interpreter treats unquoted strings as symbols instead of strings.
        The string must contain no whitespace for this to work.

    ..  container:: example

        **Example 6.** A Scheme expression of LilyPond functions:

        ::

            >>> function_1 = 'tuplet-number::append-note-wrapper'
            >>> function_2 = 'tuplet-number::calc-denominator-text'
            >>> string = schemetools.Scheme('4', force_quotes=True)
            >>> scheme = schemetools.Scheme(
            ...     function_1,
            ...     function_2,
            ...     string,
            ...     )
            >>> scheme
            Scheme('tuplet-number::append-note-wrapper', 'tuplet-number::calc-denominator-text', Scheme('4', force_quotes=True))
            >>> print(format(scheme))
            #(tuplet-number::append-note-wrapper tuplet-number::calc-denominator-text "4")

    ..  container:: example

        **Example 7.** A Scheme lambda expression of LilyPond function that
        takes a markup with a quoted string argument. Setting verbatim to true
        causes the expression to format exactly as-is without modifying quotes
        or whitespace:

        ::

            >>> string = '(lambda (grob) (grob-interpret-markup grob'
            >>> string += r' #{ \markup \musicglyph #"noteheads.s0harmonic" #}))'
            >>> scheme = schemetools.Scheme(string, verbatim=True)
            >>> scheme
            Scheme('(lambda (grob) (grob-interpret-markup grob #{ \\markup \\musicglyph #"noteheads.s0harmonic" #}))')
            >>> print(format(scheme))
            #(lambda (grob) (grob-interpret-markup grob #{ \markup \musicglyph #"noteheads.s0harmonic" #}))

    Scheme objects are immutable.
    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_force_quotes',
        '_quoting',
        '_value',
        '_verbatim',
        )

    ### INITIALIZER ###

    def __init__(self, *arguments, **keywords):
        if len(arguments) == 1:
            if isinstance(arguments[0], type(self)):
                value = arguments[0]._value
            else:
                value = arguments[0]
        else:
            value = arguments
        quoting = keywords.get('quoting')
        force_quotes = keywords.get('force_quotes')
        verbatim = keywords.get('verbatim')
        if quoting is not None:
            assert isinstance(quoting, str)
            assert all(character in r"',@`#" for character in quoting)
        if force_quotes is not None:
            force_quotes = bool(force_quotes)
        if verbatim is not None:
            verbatim = bool(verbatim)
        self._value = value
        self._quoting = quoting
        self._force_quotes = force_quotes
        self._verbatim = verbatim

    ### SPECIAL METHODS ###

    def __format__(self, format_specification=''):
        r'''Formats scheme.

        Set `format_specification` to `''`', `'lilypond'` or ``'storage'``.
        Interprets `''` equal to `'lilypond'`.

        ..  container:: example

            **Example 1.** Scheme LilyPond format:

            ::

                >>> scheme = schemetools.Scheme('foo')
                >>> format(scheme)
                '#foo'

        ..  container:: example

            **Example 2.** Scheme storage format:

            ::

                >>> print(format(scheme, 'storage'))
                schemetools.Scheme(
                    'foo'
                    )

        Returns string.
        '''
        from abjad.tools import systemtools
        if format_specification in ('', 'lilypond'):
            return self._get_lilypond_format()
        elif format_specification == 'storage':
            return systemtools.StorageFormatAgent(self).get_storage_format()
        return str(self)

    def __getnewargs__(self):
        r'''Gets new arguments.

        Returns tuple.
        '''
        return (self._value,)

    def __str__(self):
        r'''String representation of scheme object.

        Returns string.
        '''
        if self._quoting is not None:
            return self._quoting + self._formatted_value
        return self._formatted_value

    ### PRIVATE PROPERTIES ###

    @property
    def _formatted_value(self):
        return Scheme.format_scheme_value(
            self._value,
            force_quotes=self.force_quotes,
            verbatim=self.verbatim,
            )

    ### PRIVATE METHODS ###

    def _get_format_specification(self):
        if stringtools.is_string(self._value):
            values = [self._value]
        elif isinstance(self._value, collections.Iterable):
            values = self._value
        else:
            values = [self._value]
        names = []
        if self.force_quotes:
            names.append('force_quotes')
        if self.quoting:
            names.append('quoting')
        return systemtools.FormatSpecification(
            client=self,
            storage_format_args_values=values,
            storage_format_kwargs_names=names,
            )

    def _get_lilypond_format(self):
        if self._quoting is not None:
            return '#' + self._quoting + self._formatted_value
        return '#' + self._formatted_value

    ### PUBLIC PROPERTIES ###

    @property
    def force_quotes(self):
        r'''Is true when quotes should be forced in output. Otherwise false.

        Returns true or false.
        '''
        return self._force_quotes

    @property
    def quoting(self):
        r'''Gets Scheme quoting string.

        Returns string.
        '''
        return self._quoting

    @property
    def verbatim(self):
        r'''Is true when formatting should format value absolutely verbatim.
        Whitespace, quotes, and all other parts of value are left intact.

        Defaults to false.

        Set to true or false.

        Returns true or false.
        '''
        return self._verbatim

    ### PUBLIC METHODS ###

    @staticmethod
    def format_embedded_scheme_value(value, force_quotes=False):
        r'''Formats `value` as an embedded Scheme value.
        '''
        from abjad.tools import datastructuretools
        if isinstance(value, datastructuretools.OrdinalConstant):
            result = '#' + repr(value).lower()
        else:
            result = Scheme.format_scheme_value(
                value, force_quotes=force_quotes)
            if (
                (isinstance(value, bool)) or
                (isinstance(value, str) and not force_quotes) or
                (isinstance(value, Scheme))
                ):
                result = '#' + result
        return result

    @staticmethod
    def format_scheme_value(value, force_quotes=False, verbatim=False):
        r'''Formats `value` as Scheme would.

        ..  container:: example

            **Example 1.** Some basic values:

            ::

                >>> schemetools.Scheme.format_scheme_value(1)
                '1'

            ::

                >>> schemetools.Scheme.format_scheme_value('foo')
                'foo'

            ::

                >>> schemetools.Scheme.format_scheme_value('bar baz')
                '"bar baz"'

            ::

                >>> schemetools.Scheme.format_scheme_value([1.5, True, False])
                '(1.5 #t #f)'

        ..  container:: example

            **Example 2.** Strings without whitespace can be forcibly quoted
            via the `force_quotes` keyword:

            ::

                >>> schemetools.Scheme.format_scheme_value(
                ...     'foo',
                ...     force_quotes=True,
                ...     )
                '"foo"'

        ..  container:: example

            **Example 3.** Set verbatim to true to format value exactly (with
            only hash preprended):

            ::

                >>> string = '(lambda (grob) (grob-interpret-markup grob'
                >>> string += r' #{ \markup \musicglyph #"noteheads.s0harmonic" #}))'
                >>> schemetools.Scheme.format_scheme_value(string, verbatim=True)
                '(lambda (grob) (grob-interpret-markup grob #{ \\markup \\musicglyph #"noteheads.s0harmonic" #}))'

        ..  container:: example

            **Example 4.** Hash symbols in strings will result in quoted output
            unless `verbatim` is True, in order to prevent LilyPond parsing
            errors:

            ::

                >>> string = '#1-finger'
                >>> schemetools.Scheme.format_scheme_value(string)
                '"#1-finger"'

            ::

                >>> schemetools.Scheme.format_scheme_value(string, verbatim=True)
                '#1-finger'

        Returns string.
        '''
        if isinstance(value, str):
            if not verbatim:
                value = value.replace('"', r'\"')
                if force_quotes or ' ' in value or '#' in value:
                    return '"{}"'.format(value)
                return value
            else:
                return value
        elif isinstance(value, bool):
            if value:
                return '#t'
            return '#f'
        elif isinstance(value, (list, tuple)):
            return '({})'.format(
                ' '.join(Scheme.format_scheme_value(x)
                    for x in value))
        elif isinstance(value, Scheme):
            return str(value)
        elif isinstance(value, type(None)):
            return '#f'
        return str(value)



# -*- coding: utf-8 -*-
from abjad.tools import indicatortools
from abjad.tools import scoretools
from abjad.tools import selectiontools
from abjad.tools.topleveltools import iterate
from abjad.tools.quantizationtools.AttackPointOptimizer \
    import AttackPointOptimizer


class NaiveAttackPointOptimizer(AttackPointOptimizer):
    r'''Concrete ``AttackPointOptimizer`` subclass which optimizes
    attack points by fusing tie leaves within logical ties with
    leaf durations decreasing monotonically.

    ``TieChains`` will be partitioned into sub-``TieChains`` if
    leaves are found with ``TempoMarks`` attached.
    '''

    ### SPECIAL METHODS ###

    def __call__(self, argument):
        r'''Calls naive attack-point optimizer.

        Returns none.
        '''
        for logical_tie in iterate(argument).by_logical_tie(reverse=True):
            sub_logical_ties = []
            current_sub_logical_tie = []
            for leaf in logical_tie:
                tempos = leaf._get_indicators(indicatortools.Tempo)
                if tempos:
                    if current_sub_logical_tie:
                        current_sub_logical_tie = \
                            selectiontools.LogicalTie(current_sub_logical_tie)
                        sub_logical_ties.append(current_sub_logical_tie)
                    current_sub_logical_tie = []
                current_sub_logical_tie.append(leaf)
            if current_sub_logical_tie:
                current_sub_logical_tie = selectiontools.LogicalTie(current_sub_logical_tie)
                sub_logical_ties.append(current_sub_logical_tie)
            for sub_logical_tie in sub_logical_ties:
                sub_logical_tie._fuse_leaves_by_immediate_parent()



# -*- coding: utf-8 -*-
from abjad import *


def test_scoretools_Measure___init___01():
    r'''Initializes measure from empty input.
    '''

    measure = Measure()

    assert measure.time_signature == TimeSignature((4, 4))
    assert not len(measure)



# -*- coding: utf-8 -*-
# warn on an outdated Python installation
import distutils.version
import platform
if not (
    distutils.version.LooseVersion('2.7.2') <
    distutils.version.LooseVersion(platform.python_version())
    ):
    print('WARNING: Abjad does not support Python versions less than 2.7.3.')
    print('WARNING: Upgrade your Python to 2.7.3 or higher.')
del platform
del distutils

# set up tab completion
try:
    import readline
    import rlcompleter
    if readline.__doc__ is not None and 'libedit' in readline.__doc__:
        readline.parse_and_bind("bind ^I rl_complete")
    else:
        readline.parse_and_bind("tab: complete")
    del readline
    del rlcompleter
except ImportError:
    pass

try:
    from quicktions import Fraction
except ImportError:
    from fractions import Fraction

# ensure that the ~/.abjad directory and friends are setup,
# and instantiate Abjad's configuration singleton
from abjad.tools.systemtools.AbjadConfiguration import AbjadConfiguration
abjad_configuration = AbjadConfiguration()
del AbjadConfiguration

# import all tools packages
from abjad.tools import *

# import some frequently used classes for direct user access
from abjad.tools.durationtools import Duration
from abjad.tools.durationtools import Multiplier
from abjad.tools.durationtools import Offset
from abjad.tools.datastructuretools import CyclicTuple
from abjad.tools.datastructuretools import TypedOrderedDict
from abjad.tools.expressiontools import Expression
from abjad.tools.indicatortools import Accelerando
from abjad.tools.indicatortools import Annotation
from abjad.tools.indicatortools import Arpeggio
from abjad.tools.indicatortools import Arrow
from abjad.tools.indicatortools import Articulation
from abjad.tools.indicatortools import BarLine
from abjad.tools.indicatortools import BendAfter
from abjad.tools.indicatortools import BowContactPoint
from abjad.tools.indicatortools import BowMotionTechnique
from abjad.tools.indicatortools import BowPressure
from abjad.tools.indicatortools import BreathMark
from abjad.tools.indicatortools import Clef
from abjad.tools.indicatortools import ColorFingering
from abjad.tools.indicatortools import Dynamic
from abjad.tools.indicatortools import Fermata
from abjad.tools.indicatortools import KeyCluster
from abjad.tools.indicatortools import KeySignature
from abjad.tools.indicatortools import LaissezVibrer
from abjad.tools.indicatortools import LilyPondCommand
from abjad.tools.indicatortools import LilyPondComment
from abjad.tools.indicatortools import LilyPondLiteral
from abjad.tools.indicatortools import LineSegment
from abjad.tools.indicatortools import MetricModulation
from abjad.tools.indicatortools import PageBreak
from abjad.tools.indicatortools import RehearsalMark
from abjad.tools.indicatortools import Repeat
from abjad.tools.indicatortools import Ritardando
from abjad.tools.indicatortools import StaffChange
from abjad.tools.indicatortools import StemTremolo
from abjad.tools.indicatortools import StringContactPoint
from abjad.tools.indicatortools import StringNumber
from abjad.tools.indicatortools import SystemBreak
from abjad.tools.indicatortools import Tempo
from abjad.tools.indicatortools import TimeSignature
from abjad.tools.indicatortools import Tremolo
from abjad.tools.indicatortools import Tuning
from abjad.tools.lilypondfiletools import LilyPondFile
from abjad.tools.markuptools import Markup
from abjad.tools.markuptools import MarkupList
from abjad.tools.mathtools import Infinity
Infinity = Infinity()
from abjad.tools.mathtools import NonreducedFraction
from abjad.tools.mathtools import NonreducedRatio
from abjad.tools.mathtools import Ratio
from abjad.tools.metertools import Meter
from abjad.tools.patterntools import Pattern
from abjad.tools.pitchtools import Inversion
from abjad.tools.pitchtools import Multiplication
from abjad.tools.pitchtools import NamedInterval
from abjad.tools.pitchtools import NamedIntervalClass
from abjad.tools.pitchtools import NamedPitch
from abjad.tools.pitchtools import NamedPitchClass
from abjad.tools.pitchtools import NumberedInterval
from abjad.tools.pitchtools import NumberedIntervalClass
from abjad.tools.pitchtools import NumberedPitch
from abjad.tools.pitchtools import NumberedPitchClass
from abjad.tools.pitchtools import PitchClassSet
from abjad.tools.pitchtools import PitchClassSegment
from abjad.tools.pitchtools import PitchSegment
from abjad.tools.pitchtools import PitchSet
from abjad.tools.pitchtools import PitchRange
from abjad.tools.pitchtools import Registration
from abjad.tools.pitchtools import Retrograde
from abjad.tools.pitchtools import Rotation
from abjad.tools.pitchtools import SetClass
from abjad.tools.pitchtools import Transposition
from abjad.tools.pitchtools import TwelveToneRow
from abjad.tools.schemetools import Scheme
from abjad.tools.schemetools import SchemeMoment
from abjad.tools.schemetools import SchemePair
from abjad.tools.schemetools import SchemeSymbol
from abjad.tools.schemetools import SchemeVector
from abjad.tools.scoretools import Chord
from abjad.tools.scoretools import Cluster
from abjad.tools.scoretools import Component
from abjad.tools.scoretools import Container
from abjad.tools.scoretools import Context
from abjad.tools.scoretools import FixedDurationTuplet
from abjad.tools.scoretools import GraceContainer
from abjad.tools.scoretools import Leaf
from abjad.tools.scoretools import Measure
from abjad.tools.scoretools import Measure
from abjad.tools.scoretools import MultimeasureRest
from abjad.tools.scoretools import Note
from abjad.tools.scoretools import Rest
from abjad.tools.scoretools import Score
from abjad.tools.scoretools import Skip
from abjad.tools.scoretools import Staff
from abjad.tools.scoretools import StaffGroup
from abjad.tools.scoretools import Tuplet
from abjad.tools.scoretools import Voice
from abjad.tools.selectiontools import Selection
from abjad.tools.selectortools import Selector
from abjad.tools.sequencetools import Sequence
from abjad.tools.spannertools import Beam
from abjad.tools.spannertools import BowContactSpanner
from abjad.tools.spannertools import ClefSpanner
from abjad.tools.spannertools import ComplexBeam
from abjad.tools.spannertools import ComplexTrillSpanner
from abjad.tools.spannertools import Crescendo
from abjad.tools.spannertools import Decrescendo
from abjad.tools.spannertools import DuratedComplexBeam
from abjad.tools.spannertools import GeneralizedBeam
from abjad.tools.spannertools import Glissando
from abjad.tools.spannertools import Hairpin
from abjad.tools.spannertools import HiddenStaffSpanner
from abjad.tools.spannertools import HorizontalBracketSpanner
from abjad.tools.spannertools import MeasuredComplexBeam
from abjad.tools.spannertools import MultipartBeam
from abjad.tools.spannertools import OctavationSpanner
from abjad.tools.spannertools import PhrasingSlur
from abjad.tools.spannertools import PianoPedalSpanner
from abjad.tools.spannertools import Slur
from abjad.tools.spannertools import Spanner
from abjad.tools.spannertools import StaffLinesSpanner
from abjad.tools.spannertools import StemTremoloSpanner
from abjad.tools.spannertools import TempoSpanner
from abjad.tools.spannertools import TextSpanner
from abjad.tools.spannertools import Tie
from abjad.tools.spannertools import TrillSpanner
from abjad.tools.timespantools import Timespan

# import some frequently used functions for direct user access
from abjad.tools.topleveltools import attach
from abjad.tools.topleveltools import detach
from abjad.tools.topleveltools import graph
from abjad.tools.topleveltools import inspect_
from abjad.tools.topleveltools import iterate
from abjad.tools.topleveltools import label
from abjad.tools.topleveltools import mutate
from abjad.tools.topleveltools import new
from abjad.tools.topleveltools import override
from abjad.tools.topleveltools import parse
from abjad.tools.topleveltools import play
from abjad.tools.topleveltools import persist
from abjad.tools.topleveltools import select
from abjad.tools.topleveltools import sequence
from abjad.tools.topleveltools import set_
from abjad.tools.topleveltools import show
from abjad.tools.topleveltools import tweak

# frequently used patterntools functions
from abjad.tools.patterntools import select_all
from abjad.tools.patterntools import select_every
from abjad.tools.patterntools import select_first
from abjad.tools.patterntools import select_last

# frequently used rhythm-maker functions
from abjad.tools.rhythmmakertools import silence
from abjad.tools.rhythmmakertools import silence_all
from abjad.tools.rhythmmakertools import silence_every
from abjad.tools.rhythmmakertools import silence_except
from abjad.tools.rhythmmakertools import silence_first
from abjad.tools.rhythmmakertools import silence_last
from abjad.tools.rhythmmakertools import sustain
from abjad.tools.rhythmmakertools import sustain_all
from abjad.tools.rhythmmakertools import sustain_every
from abjad.tools.rhythmmakertools import sustain_first
from abjad.tools.rhythmmakertools import sustain_last

# import custom exceptions into the builtins module
import os
from abjad.tools.systemtools.ImportManager import ImportManager
ImportManager.import_public_names_from_path_into_namespace(
    os.path.join(__path__[0], 'tools', 'exceptiontools'),
    __builtins__,
    )
del ImportManager
del os
del tools

# import version information
from abjad._version import __version_info__, __version__
del _version

def f(argument):
    if hasattr(argument, '_publish_storage_format'):
        print(format(argument, 'storage'))
    else:
        print(format(argument, 'lilypond'))

from abjad import demos
from abjad import ext
from abjad import ly



# -*- coding: utf-8 -*-
from abjad import *


def test_systemtools_AbjadConfiguration_get_python_version_string_01():
    python_version_string = \
        systemtools.AbjadConfiguration.get_python_version_string()
    assert isinstance(python_version_string, str)
    assert python_version_string.count('.') == 2



# -*- coding: utf-8 -*-
from abjad.tools import lilypondparsertools


def test_lilypondparsertools_LilyPondParser_accidentals_forced_01():

    string = "{ c!4 }"
    parsed = lilypondparsertools.LilyPondParser()(string)

    assert parsed[0].note_head.is_forced == True
    assert format(parsed[0]) == 'c!4'


def test_lilypondparsertools_LilyPondParser_accidentals_forced_02():

    string = "{ <c! e g!!>4 }"
    parsed = lilypondparsertools.LilyPondParser()(string)

    assert parsed[0].note_heads[0].is_forced == True
    assert parsed[0].note_heads[1].is_forced == False
    assert parsed[0].note_heads[2].is_forced == True
    assert format(parsed[0]) == '<c! e g!>4'



# -*- coding: utf-8 -*-
from abjad import *
from abjad.tools import tonalanalysistools


def test_tonalanalysistools_ChordSuspension__initialize_by_symbolic_string_01():

    chord_suspension = tonalanalysistools.ChordSuspension('4-3')
    assert chord_suspension.start == tonalanalysistools.ScaleDegree(4)
    assert chord_suspension.stop == tonalanalysistools.ScaleDegree(3)

    chord_suspension = tonalanalysistools.ChordSuspension('b2-1')
    assert chord_suspension.start == tonalanalysistools.ScaleDegree('flat', 2)
    assert chord_suspension.stop == tonalanalysistools.ScaleDegree(1)



# -*- coding: utf-8 -*-
from abjad import *


def test_systemtools_TestManager_get_current_function_name_01():

    def bar():
        return systemtools.TestManager.get_current_function_name()

    assert bar() == 'bar'


def test_systemtools_TestManager_get_current_function_name_02():

    def foo():
        return systemtools.TestManager.get_current_function_name()

    assert foo() == 'foo'



# -*- coding: utf-8 -*-
from abjad import *


def test_pitchtools_PitchSet_is_pitch_class_unique_01():

    pitch_set = pitchtools.PitchSet([0, 13, 26])
    assert pitch_set.is_pitch_class_unique


def test_pitchtools_PitchSet_is_pitch_class_unique_02():

    pitch_set = pitchtools.PitchSet([0, 12, 13, 26])
    assert not pitch_set.is_pitch_class_unique


def test_pitchtools_PitchSet_is_pitch_class_unique_03():
    r'''Empty pitch-set and length-1 pitch-set boundary cases.
    '''

    assert pitchtools.PitchSet([]).is_pitch_class_unique
    assert pitchtools.PitchSet([13]).is_pitch_class_unique



# -*- coding: utf-8 -*-
from abjad.tools import systemtools
import pytest
try:
    from unittest import mock
except ImportError:
    import mock

midi_player, text_editor, pdf_viewer = (
    'midi_player_application',
    'text_editor_application',
    'pdf_viewer_application',
)
test_files_to_viewer_mapping = {
    'test.midi': midi_player,
    'test.py': text_editor,
    'test.txt': text_editor,
    'test.pdf': pdf_viewer,
}
test_files = test_files_to_viewer_mapping.keys()
empty_abjad_configuration, nonempty_abjad_configuration = (
    (
        'abjad.abjad_configuration', {
            'midi_player': None,
            'text_editor': None,
            'pdf_viewer': None,
        }
    ),
    (
        'abjad.abjad_configuration', {
            'midi_player': midi_player,
            'text_editor': text_editor,
            'pdf_viewer': pdf_viewer,
        },
    )
)
abjad_configurations = (
    empty_abjad_configuration,
    nonempty_abjad_configuration,
)
applications = (None, 'custom-viewer')


@mock.patch('sys.platform', 'linux2')
@mock.patch('abjad.systemtools.IOManager.spawn_subprocess')
@pytest.mark.parametrize('configuration', abjad_configurations)
@pytest.mark.parametrize('file_path', test_files)
@pytest.mark.parametrize('application', applications)
def test_systemtools_IOManager_open_file_01(spawn_subprocess_mock,
                                             configuration,
                                             file_path,
                                             application):
    with mock.patch(*configuration):
        systemtools.IOManager.open_file(
            file_path=file_path,
            application=application,
        )
        command = None
        if application is None:
            command = 'xdg-open {}'.format(file_path)
        else:
            command = '{} {}'.format(application, file_path)
        spawn_subprocess_mock.assert_called_with(command)


@mock.patch('sys.platform', 'darwin')
@mock.patch('abjad.systemtools.IOManager.spawn_subprocess')
@pytest.mark.parametrize('configuration', abjad_configurations)
@pytest.mark.parametrize('file_path', test_files)
@pytest.mark.parametrize('application', applications)
def test_systemtools_IOManager_open_file_02(spawn_subprocess_mock,
                                             configuration,
                                             file_path,
                                             application):
    with mock.patch(*configuration):
        systemtools.IOManager.open_file(
            file_path=file_path,
            application=application,
        )
        is_empty_config = configuration == empty_abjad_configuration
        command = None
        if application is None and is_empty_config:
            command = 'open {}'.format(file_path)
        elif application is None and not is_empty_config:
            application_ = test_files_to_viewer_mapping[file_path]
            command = '{} {}'.format(application_, file_path)
        else:
            command = '{} {}'.format(application, file_path)
        spawn_subprocess_mock.assert_called_with(command)


@mock.patch('sys.platform', 'win32')
@mock.patch('os.startfile', create=True)
@pytest.mark.parametrize('configuration', abjad_configurations)
@pytest.mark.parametrize('file_path', test_files)
@pytest.mark.parametrize('application', applications)
def test_systemtools_IOManager_open_file_03(startfile_mock,
                                             configuration,
                                             file_path,
                                             application):
    with mock.patch(*configuration):
        systemtools.IOManager.open_file(
            file_path=file_path,
            application=application,
        )
        startfile_mock.assert_called_with(file_path)



# -*- coding: utf-8 -*-
from abjad import *


def test_instrumenttools_Instrument__default_instrument_name_to_instrument_class_01():

    instrument_class = \
        instrumenttools.Instrument._default_instrument_name_to_instrument_class(
            'clarinet in E-flat')
    assert instrument_class == instrumenttools.ClarinetInEFlat


def test_instrumenttools_Instrument__default_instrument_name_to_instrument_class_02():

    assert instrumenttools.Instrument._default_instrument_name_to_instrument_class(
        'foo') is None



# -*- coding: utf-8 -*-
from abjad.tools import indicatortools
from abjad.tools import durationtools
from abjad.tools.quantizationtools.QSchemaItem import QSchemaItem


class MeasurewiseQSchemaItem(QSchemaItem):
    '''`MeasurewiseQSchemaItem` represents a change of state in the
    timeline of a metered quantization process.

    ::

        >>> q_schema_item = quantizationtools.MeasurewiseQSchemaItem()
        >>> print(format(q_schema_item))
        quantizationtools.MeasurewiseQSchemaItem()

    Define a change in tempo:

    ::

        >>> q_schema_item = quantizationtools.MeasurewiseQSchemaItem(
        ...     tempo=((1, 4), 60),
        ...     )
        >>> print(format(q_schema_item))
        quantizationtools.MeasurewiseQSchemaItem(
            tempo=indicatortools.Tempo(
                reference_duration=durationtools.Duration(1, 4), 
                units_per_minute=60,
                ),
            )

    Define a change in time signature:

    ::

        >>> q_schema_item = quantizationtools.MeasurewiseQSchemaItem(
        ...     time_signature=(6, 8),
        ...     )
        >>> print(format(q_schema_item))
        quantizationtools.MeasurewiseQSchemaItem(
            time_signature=indicatortools.TimeSignature((6, 8)),
            )

    Test for beatspan, given a defined time signature:

    ::

        >>> q_schema_item.beatspan
        Duration(1, 8)

    `MeasurewiseQSchemaItem` is immutable.

    Return `MeasurewiseQSchemaItem` instance.
    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_time_signature',
        '_use_full_measure',
        )

    ### INITIALIZER ###

    def __init__(
        self,
        search_tree=None,
        tempo=None,
        time_signature=None,
        use_full_measure=None
        ):
        QSchemaItem.__init__(
            self,
            search_tree=search_tree,
            tempo=tempo,
            )
        if time_signature is not None:
            time_signature = indicatortools.TimeSignature(time_signature)
        self._time_signature = time_signature
        if use_full_measure is not None:
            use_full_measure = bool(use_full_measure)
        self._use_full_measure = use_full_measure

    ### PUBLIC PROPERTIES ###

    @property
    def beatspan(self):
        r'''The beatspan duration, if a time signature was defined.

        Returns duration or none.
        '''
        if self.time_signature is not None:
            if self.use_full_measure:
                return self.time_signature.duration
            else:
                return durationtools.Duration(
                    1, self.time_signature.denominator)
        return None

    @property
    def time_signature(self):
        r'''The optionally defined TimeSignature.

        Returns time signature or none
        '''
        return self._time_signature

    @property
    def use_full_measure(self):
        r'''If True, use the full measure as the beatspan.

        Returns boolean or none.
        '''
        return self._use_full_measure



# -*- coding: utf-8 -*-
from abjad import *


def test_scoretools_extend_measures_and_apply_full_measure_tuplets_01():

    staff = Staff([Measure((2, 8), "c'8 d'8"), Measure((3, 8), "e'8 f'8 g'8")])

    r'''
    \new Staff {
        {
            \time 2/8
            c'8
            d'8
        }
        {
            \time 3/8
            e'8
            f'8
            g'8
        }
    }
    '''
    supplement = [Rest((1, 16))]
    result = scoretools.extend_measures_and_apply_full_measure_tuplets(staff, supplement)

    r'''
    \new Staff {
        {
            \time 2/8
            \times 4/5 {
                c'8
                d'8
                r16
            }
        }
        {
            \time 3/8
            \tweak text #tuplet-number::calc-fraction-text
            \times 6/7 {
                e'8
                f'8
                g'8
                r16
            }
        }
    }
    '''

    assert inspect_(staff).is_well_formed()
    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 2/8
                \times 4/5 {
                    c'8
                    d'8
                    r16
                }
            }
            {
                \time 3/8
                \tweak text #tuplet-number::calc-fraction-text
                \times 6/7 {
                    e'8
                    f'8
                    g'8
                    r16
                }
            }
        }
        '''
        )



# -*- coding: utf-8 -*-
import os
from abjad import *
configuration = systemtools.AbjadConfiguration()

path_1 = os.path.join(
    configuration.abjad_directory,
    'test_1.py',
    )
path_2 = os.path.join(
    configuration.abjad_directory,
    'test_2.py',
    )


def test_systemtools_TestManager__compare_text_files_01():
    r'''Is true when lines are exactly the same.
    '''

    with systemtools.FilesystemState(remove=[path_1, path_2]):
        first_lines = ["print 'hello'"]
        second_lines = first_lines[:]
        with open(path_1, 'w') as file_pointer:
            file_pointer.write('\n'.join(first_lines))
        with open(path_2, 'w') as file_pointer:
            file_pointer.write('\n'.join(second_lines))
        assert systemtools.TestManager._compare_text_files(path_1, path_2)


def test_systemtools_TestManager__compare_text_files_02():
    r'''Is true when white space differs.
    '''

    with systemtools.FilesystemState(remove=[path_1, path_2]):
        first_lines = ["print 'hello'", '', '', "print 'goodbye'", '']
        second_lines = ['', "print 'hello'", "print 'goodbye'"]
        with open(path_1, 'w') as file_pointer:
            file_pointer.write('\n'.join(first_lines))
        with open(path_2, 'w') as file_pointer:
            file_pointer.write('\n'.join(second_lines))
        assert systemtools.TestManager._compare_text_files(path_1, path_2)


def test_systemtools_TestManager__compare_text_files_03():
    r'''False when any other lines differ.
    '''

    with systemtools.FilesystemState(remove=[path_1, path_2]):
        first_lines = ["print 'hello'"]
        second_lines = ["print 'goodbye'"]
        with open(path_1, 'w') as file_pointer:
            file_pointer.write('\n'.join(first_lines))
        with open(path_2, 'w') as file_pointer:
            file_pointer.write('\n'.join(second_lines))
        assert not systemtools.TestManager._compare_text_files(path_1, path_2)



# -*- coding: utf-8 -*-
from abjad import *


def test_scoretools_Tuplet_set_minimum_denominator_01():

    tuplet = Tuplet(Multiplier(3, 5), "c'4 d'8 e'8 f'4 g'2")
    tuplet.set_minimum_denominator(8)

    assert format(tuplet) == stringtools.normalize(
        r'''
        \tweak text #tuplet-number::calc-fraction-text
        \times 6/10 {
            c'4
            d'8
            e'8
            f'4
            g'2
        }
        '''
        )

    assert inspect_(tuplet).is_well_formed()


def test_scoretools_Tuplet_set_minimum_denominator_02():

    tuplet = Tuplet(Multiplier(3, 5), "c'4 d'8 e'8 f'4 g'2")
    tuplet.set_minimum_denominator(16)

    assert format(tuplet) == stringtools.normalize(
        r'''
        \tweak text #tuplet-number::calc-fraction-text
        \times 12/20 {
            c'4
            d'8
            e'8
            f'4
            g'2
        }
        '''
        )

    assert inspect_(tuplet).is_well_formed()


def test_scoretools_Tuplet_set_minimum_denominator_03():

    tuplet = Tuplet(Multiplier(3, 5), "c'4 d'8 e'8 f'4 g'2")
    tuplet.set_minimum_denominator(2)

    assert format(tuplet) == stringtools.normalize(
        r'''
        \tweak text #tuplet-number::calc-fraction-text
        \times 3/5 {
            c'4
            d'8
            e'8
            f'4
            g'2
        }
        '''
        )

    assert inspect_(tuplet).is_well_formed()



# -*- coding: utf-8 -*-
from abjad import *


def test_systemtools_AbjadConfiguration_list_package_dependency_versions_01():

    deps = systemtools.AbjadConfiguration.list_package_dependency_versions()
    assert isinstance(deps, dict)
    assert 'pytest' in deps
    assert isinstance(deps['pytest'], (type(None), str))
    assert 'sphinx' in deps
    assert isinstance(deps['sphinx'], (type(None), str))



# -*- coding: utf-8 -*-
from abjad.tools.abctools.AbjadValueObject import AbjadValueObject


class KeyCluster(AbjadValueObject):
    r'''Key cluster.

    ..  container:: example

        
        **Example 1.** Default values:

        ::

            >>> chord = Chord("<c' e' g' b' d'' f''>8")
            >>> key_cluster = indicatortools.KeyCluster()
            >>> attach(key_cluster, chord)
            >>> show(chord) # doctest: +SKIP

        ..  doctest::

            >>> print(format(chord))
            \once \override Accidental.stencil = ##f
            \once \override AccidentalCautionary.stencil = ##f
            \once \override Arpeggio.X-offset = #-2
            \once \override NoteHead.stencil = #ly:text-interface::print
            \once \override NoteHead.text = \markup {
                \filled-box #'(-0.6 . 0.6) #'(-0.7 . 0.7) #0.25
            }
            <c' e' g' b' d'' f''>8
                ^ \markup {
                    \center-align
                        \concat
                            {
                                \natural
                                \flat
                            }
                    }

    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_default_scope',
        '_include_black_keys',
        '_include_white_keys',
        '_markup_direction',
        '_suppress_markup',
        )

    ### INITIALIZER ###

    def __init__(
        self,
        include_black_keys=True,
        include_white_keys=True,
        markup_direction=Up,
        suppress_markup=False,
        ):
        self._default_scope = None
        assert include_black_keys or include_white_keys
        self._include_black_keys = bool(include_black_keys)
        self._include_white_keys = bool(include_white_keys)
        assert markup_direction in (Up, Center, Down)
        self._markup_direction = markup_direction
        self._suppress_markup = bool(suppress_markup)

    ### PRIVATE METHODS ###

    def _get_lilypond_format_bundle(self, component=None):
        from abjad.tools import markuptools
        from abjad.tools import systemtools
        lilypond_format_bundle = systemtools.LilyPondFormatBundle()
        lilypond_format_bundle.grob_overrides.append(
            '\\once \\override Accidental.stencil = ##f\n'
            '\\once \\override AccidentalCautionary.stencil = ##f\n'
            '\\once \\override Arpeggio.X-offset = #-2\n'
            '\\once \\override NoteHead.stencil = #ly:text-interface::print\n'
            '\\once \\override NoteHead.text = \markup {\n'
            "\t\\filled-box #'(-0.6 . 0.6) #'(-0.7 . 0.7) #0.25\n"
            '}'
            )
        if not self.suppress_markup:
            if self.include_black_keys and self.include_white_keys:
                string = r'\center-align \concat { \natural \flat }'
            elif self.include_black_keys:
                string = r'\center-align \flat'
            else:
                string = r'\center-align \natural'
            markup = markuptools.Markup(
                string,
                direction=self.markup_direction,
                )
            markup_format_pieces = markup._get_format_pieces()
            lilypond_format_bundle.right.markup.extend(markup_format_pieces)
        return lilypond_format_bundle

    ### PUBLIC PROPERTIES ###

    @property
    def default_scope(self):
        r'''Gest default scope of key cluster indicator.

        ..  container:: example

            ::

                >>> key_cluster = indicatortools.KeyCluster()
                >>> key_cluster.default_scope is None
                True

        Returns none.
        '''
        return self._default_scope

    @property
    def include_black_keys(self):
        r'''Is true if key cluster includes black keys.

        ..  container:: example

            **Example 1.** Includes flat markup:

            ::

                >>> chord = Chord("<c' e' g' b' d'' f''>8")
                >>> key_cluster = indicatortools.KeyCluster(
                ...     include_black_keys=True,
                ...     )
                >>> attach(key_cluster, chord)
                >>> show(chord) # doctest: +SKIP

            ..  doctest::

                >>> print(format(chord))
                \once \override Accidental.stencil = ##f
                \once \override AccidentalCautionary.stencil = ##f
                \once \override Arpeggio.X-offset = #-2
                \once \override NoteHead.stencil = #ly:text-interface::print
                \once \override NoteHead.text = \markup {
                    \filled-box #'(-0.6 . 0.6) #'(-0.7 . 0.7) #0.25
                }
                <c' e' g' b' d'' f''>8
                    ^ \markup {
                        \center-align
                            \concat
                                {
                                    \natural
                                    \flat
                                }
                        }

            Default behavior.

        ..  container:: example

            **Example 2.** Does not include flat markup:

            ::

                >>> chord = Chord("<c' e' g' b' d'' f''>8")
                >>> key_cluster = indicatortools.KeyCluster(
                ...     include_black_keys=False,
                ...     )
                >>> attach(key_cluster, chord)
                >>> show(chord) # doctest: +SKIP

            ..  doctest::

                >>> print(format(chord))
                \once \override Accidental.stencil = ##f
                \once \override AccidentalCautionary.stencil = ##f
                \once \override Arpeggio.X-offset = #-2
                \once \override NoteHead.stencil = #ly:text-interface::print
                \once \override NoteHead.text = \markup {
                    \filled-box #'(-0.6 . 0.6) #'(-0.7 . 0.7) #0.25
                }
                <c' e' g' b' d'' f''>8
                    ^ \markup {
                        \center-align
                            \natural
                        }

        ..  todo:: Rename to ``include_flat_markup``.

        Set to true or false.
        '''
        return self._include_black_keys

    @property
    def include_white_keys(self):
        r'''Is true if key cluster includes white keys.

        ..  container:: example

            **Example 1.** Includes natural markup:

            ::

                >>> chord = Chord("<c' e' g' b' d'' f''>8")
                >>> key_cluster = indicatortools.KeyCluster(
                ...     include_white_keys=True,
                ...     )
                >>> attach(key_cluster, chord)
                >>> show(chord) # doctest: +SKIP

            ..  doctest::

                >>> print(format(chord))
                \once \override Accidental.stencil = ##f
                \once \override AccidentalCautionary.stencil = ##f
                \once \override Arpeggio.X-offset = #-2
                \once \override NoteHead.stencil = #ly:text-interface::print
                \once \override NoteHead.text = \markup {
                    \filled-box #'(-0.6 . 0.6) #'(-0.7 . 0.7) #0.25
                }
                <c' e' g' b' d'' f''>8
                    ^ \markup {
                        \center-align
                            \concat
                                {
                                    \natural
                                    \flat
                                }
                        }

            Default behavior.

        ..  container:: example

            **Example 2.** Does not include natural markup:

            ::

                >>> chord = Chord("<c' e' g' b' d'' f''>8")
                >>> key_cluster = indicatortools.KeyCluster(
                ...     include_white_keys=False,
                ...     )
                >>> attach(key_cluster, chord)
                >>> show(chord) # doctest: +SKIP

            ..  doctest::

                >>> print(format(chord))
                \once \override Accidental.stencil = ##f
                \once \override AccidentalCautionary.stencil = ##f
                \once \override Arpeggio.X-offset = #-2
                \once \override NoteHead.stencil = #ly:text-interface::print
                \once \override NoteHead.text = \markup {
                    \filled-box #'(-0.6 . 0.6) #'(-0.7 . 0.7) #0.25
                }
                <c' e' g' b' d'' f''>8
                    ^ \markup {
                        \center-align
                            \flat
                        }

        ..  todo:: Rename to ``include_natural_markup``.

        Set to true or false.
        '''
        return self._include_white_keys

    @property
    def markup_direction(self):
        r'''Gets markup direction.

        ..  container:: example

            **Example 1.** Positions markup up:

            ::

                >>> chord = Chord("<c' e' g' b' d'' f''>8")
                >>> key_cluster = indicatortools.KeyCluster(
                ...     markup_direction=Up,
                ...     )
                >>> attach(key_cluster, chord)
                >>> show(chord) # doctest: +SKIP

            ..  doctest::

                >>> print(format(chord))
                \once \override Accidental.stencil = ##f
                \once \override AccidentalCautionary.stencil = ##f
                \once \override Arpeggio.X-offset = #-2
                \once \override NoteHead.stencil = #ly:text-interface::print
                \once \override NoteHead.text = \markup {
                    \filled-box #'(-0.6 . 0.6) #'(-0.7 . 0.7) #0.25
                }
                <c' e' g' b' d'' f''>8
                    ^ \markup {
                        \center-align
                            \concat
                                {
                                    \natural
                                    \flat
                                }
                        }

            Default behavior.

        ..  container:: example

            **Example 2.** Positions markup down:

            ::

                >>> chord = Chord("<c' e' g' b' d'' f''>8")
                >>> key_cluster = indicatortools.KeyCluster(
                ...     markup_direction=Down,
                ...     )
                >>> attach(key_cluster, chord)
                >>> show(chord) # doctest: +SKIP

            ..  doctest::

                >>> print(format(chord))
                \once \override Accidental.stencil = ##f
                \once \override AccidentalCautionary.stencil = ##f
                \once \override Arpeggio.X-offset = #-2
                \once \override NoteHead.stencil = #ly:text-interface::print
                \once \override NoteHead.text = \markup {
                    \filled-box #'(-0.6 . 0.6) #'(-0.7 . 0.7) #0.25
                }
                <c' e' g' b' d'' f''>8
                    _ \markup {
                        \center-align
                            \concat
                                {
                                    \natural
                                    \flat
                                }
                        }

        Set to up, down or center.
        '''
        return self._markup_direction

    @property
    def suppress_markup(self):
        r'''Is true if key cluster suppresses key markup.

        ..  container:: example

            **Example 1.** Does not suppress markup:

            ::

                >>> chord = Chord("<c' e' g' b' d'' f''>8")
                >>> key_cluster = indicatortools.KeyCluster(
                ...     suppress_markup=False,
                ...     )
                >>> attach(key_cluster, chord)
                >>> show(chord) # doctest: +SKIP

            ..  doctest::

                >>> print(format(chord))
                \once \override Accidental.stencil = ##f
                \once \override AccidentalCautionary.stencil = ##f
                \once \override Arpeggio.X-offset = #-2
                \once \override NoteHead.stencil = #ly:text-interface::print
                \once \override NoteHead.text = \markup {
                    \filled-box #'(-0.6 . 0.6) #'(-0.7 . 0.7) #0.25
                }
                <c' e' g' b' d'' f''>8
                    ^ \markup {
                        \center-align
                            \concat
                                {
                                    \natural
                                    \flat
                                }
                        }

            Default behavior.

        ..  container:: example

            **Example 2.** Does not suppress markup:

            ::

                >>> chord = Chord("<c' e' g' b' d'' f''>8")
                >>> key_cluster = indicatortools.KeyCluster(
                ...     suppress_markup=True,
                ...     )
                >>> attach(key_cluster, chord)
                >>> show(chord) # doctest: +SKIP

            ..  doctest::

                >>> print(format(chord))
                \once \override Accidental.stencil = ##f
                \once \override AccidentalCautionary.stencil = ##f
                \once \override Arpeggio.X-offset = #-2
                \once \override NoteHead.stencil = #ly:text-interface::print
                \once \override NoteHead.text = \markup {
                    \filled-box #'(-0.6 . 0.6) #'(-0.7 . 0.7) #0.25
                }
                <c' e' g' b' d'' f''>8

        ..  todo:: Remove?

        Set to true or false.
        '''
        return self._suppress_markup



# -*- coding: utf-8 -*-
from abjad import *


def test_spannertools_Glissando___init___01():
    r'''Initialize empty glissando spanner.
    '''

    glissando = spannertools.Glissando()
    assert isinstance(glissando, spannertools.Glissando)



# -*- coding: utf-8 -*-


def set_(argument):
    r'''Makes LilyPond setting name manager.

    ..  container:: example

        Sets instrument name:

        ::

            >>> staff = Staff("c'4 e'4 d'4 f'4")
            >>> set_(staff).instrument_name = Markup('Vn. I')
            >>> show(staff) # doctest: +SKIP


        ..  doctest::

            >>> f(staff)
            \new Staff \with {
                instrumentName = \markup { "Vn. I" }
            } {
                c'4
                e'4
                d'4
                f'4
            }

    ..  container:: example

        Returns LilyPond setting name manager:

        ::

            >>> set_(staff)
            LilyPondSettingNameManager(('instrument_name', Markup(contents=['Vn. I'])))

    '''
    from abjad.tools import lilypondnametools
    if getattr(argument, '_lilypond_setting_name_manager', None) is None:
        manager = lilypondnametools.LilyPondSettingNameManager()
        argument._lilypond_setting_name_manager = manager
    return argument._lilypond_setting_name_manager



# -*- coding: utf-8 -*-
import sys
import unittest
from abjad.tools import abjadbooktools
from abjad.tools import stringtools


class CodeBlockTests(unittest.TestCase):

    def test_as_docutils_01(self):
        code_block = abjadbooktools.CodeBlock(())
        code_block.output_proxies.append(
            abjadbooktools.CodeOutputProxy(
                (
                    '>>> for i in range(4):',
                    '...     print(i)',
                    '0',
                    '1',
                    '2',
                    '3',
                    '>>> 1 + 1',
                    '2',
                    ),
                ),
            )
        result = code_block.as_docutils()
        self.assertEqual(len(result), 2)
        self.assertEqual(
            stringtools.normalize(result[0].pformat()),
            stringtools.normalize(
                r'''
                <literal_block xml:space="preserve">
                    >>> for i in range(4):
                    ...     print(i)
                    0
                    1
                    2
                    3
                '''),
            )
        self.assertEqual(
            stringtools.normalize(result[1].pformat()),
            stringtools.normalize(
                r'''
                <literal_block xml:space="preserve">
                    >>> 1 + 1
                    2
                '''),
            )

    def test_from_docutils_abjad_import_block_01(self):
        source = '''
        ..  import:: abjad.tools.abjadbooktools:example_function
        '''
        source = stringtools.normalize(source)
        document = abjadbooktools.SphinxDocumentHandler.parse_rst(source)
        block = document[0]
        result = abjadbooktools.CodeBlock.from_docutils_abjad_import_block(block)
        if sys.version_info[0] == 2:
            assert format(result) == stringtools.normalize(r"""
                abjadbooktools.CodeBlock(
                    (
                        u'def example_function(argument):',
                        u"    r'''This is a multiline docstring.",
                        u'',
                        u'    This is the third line of the docstring.',
                        u"    '''",
                        u'    # This is a comment.',
                        u"    print('Entering example function.')",
                        u'    try:',
                        u'        argument = argument + 1',
                        u'    except TypeError:',
                        u"        print('Wrong type!')",
                        u'    print(argument)',
                        u"    print('Leaving example function.')",
                        ),
                    executed_lines=(
                        'from abjad.tools.abjadbooktools import example_function',
                        ),
                    starting_line_number=1,
                    )
                """)
        else:            
            assert format(result) == stringtools.normalize(r"""
                abjadbooktools.CodeBlock(
                    (
                        'def example_function(argument):',
                        "    r'''This is a multiline docstring.",
                        '',
                        '    This is the third line of the docstring.',
                        "    '''",
                        '    # This is a comment.',
                        "    print('Entering example function.')",
                        '    try:',
                        '        argument = argument + 1',
                        '    except TypeError:',
                        "        print('Wrong type!')",
                        '    print(argument)',
                        "    print('Leaving example function.')",
                        ),
                    executed_lines=(
                        'from abjad.tools.abjadbooktools import example_function',
                        ),
                    starting_line_number=1,
                    )
                """)

    def test_from_docutils_abjad_import_block_02(self):
        source = '''
        ..  import:: abjad.tools.abjadbooktools:example_function
            :hide:
        '''
        source = stringtools.normalize(source)
        document = abjadbooktools.SphinxDocumentHandler.parse_rst(source)
        block = document[0]
        result = abjadbooktools.CodeBlock.from_docutils_abjad_import_block(block)
        if sys.version_info[0] == 2:
            assert format(result) == stringtools.normalize(r"""
                abjadbooktools.CodeBlock(
                    (
                        u'def example_function(argument):',
                        u"    r'''This is a multiline docstring.",
                        u'',
                        u'    This is the third line of the docstring.',
                        u"    '''",
                        u'    # This is a comment.',
                        u"    print('Entering example function.')",
                        u'    try:',
                        u'        argument = argument + 1',
                        u'    except TypeError:',
                        u"        print('Wrong type!')",
                        u'    print(argument)',
                        u"    print('Leaving example function.')",
                        ),
                    code_block_specifier=abjadbooktools.CodeBlockSpecifier(
                        hide=True,
                        ),
                    executed_lines=(
                        'from abjad.tools.abjadbooktools import example_function',
                        ),
                    starting_line_number=1,
                    )
                """)
        else:
            assert format(result) == stringtools.normalize(r"""
                abjadbooktools.CodeBlock(
                    (
                        'def example_function(argument):',
                        "    r'''This is a multiline docstring.",
                        '',
                        '    This is the third line of the docstring.',
                        "    '''",
                        '    # This is a comment.',
                        "    print('Entering example function.')",
                        '    try:',
                        '        argument = argument + 1',
                        '    except TypeError:',
                        "        print('Wrong type!')",
                        '    print(argument)',
                        "    print('Leaving example function.')",
                        ),
                    code_block_specifier=abjadbooktools.CodeBlockSpecifier(
                        hide=True,
                        ),
                    executed_lines=(
                        'from abjad.tools.abjadbooktools import example_function',
                        ),
                    starting_line_number=1,
                    )
                """)

    def test_from_docutils_abjad_input_block_01(self):
        source = '''
        ..  abjad::

            note = Note("c'4")
            if True:
                note.written_pitch = "ds,"
        '''
        source = stringtools.normalize(source)
        document = abjadbooktools.SphinxDocumentHandler.parse_rst(source)
        block = document[0]
        result = abjadbooktools.CodeBlock.from_docutils_abjad_input_block(block)
        if sys.version_info[0] == 2:
            assert format(result) == stringtools.normalize(r"""
                abjadbooktools.CodeBlock(
                    (
                        u'note = Note("c\'4")',
                        u'if True:',
                        u'    note.written_pitch = "ds,"',
                        ),
                    starting_line_number=2,
                    )
                """)
        else:
            assert format(result) == stringtools.normalize(r"""
                abjadbooktools.CodeBlock(
                    (
                        'note = Note("c\'4")',
                        'if True:',
                        '    note.written_pitch = "ds,"',
                        ),
                    starting_line_number=2,
                    )
                """)

    def test_from_docutils_abjad_input_block_02(self):
        source = '''
        ..  abjad::
            :allow-exceptions:

            note = Note("c'4")
            if True:
                note.written_pitch = "ds,"
        '''
        source = stringtools.normalize(source)
        document = abjadbooktools.SphinxDocumentHandler.parse_rst(source)
        block = document[0]
        result = abjadbooktools.CodeBlock.from_docutils_abjad_input_block(block)
        if sys.version_info[0] == 2:
            assert format(result) == stringtools.normalize(r"""
                abjadbooktools.CodeBlock(
                    (
                        u'note = Note("c\'4")',
                        u'if True:',
                        u'    note.written_pitch = "ds,"',
                        ),
                    code_block_specifier=abjadbooktools.CodeBlockSpecifier(
                        allow_exceptions=True,
                        ),
                    starting_line_number=3,
                    )
                """)
        else:
            assert format(result) == stringtools.normalize(r"""
                abjadbooktools.CodeBlock(
                    (
                        'note = Note("c\'4")',
                        'if True:',
                        '    note.written_pitch = "ds,"',
                        ),
                    code_block_specifier=abjadbooktools.CodeBlockSpecifier(
                        allow_exceptions=True,
                        ),
                    starting_line_number=3,
                    )
                """)

    def test_from_docutils_literal_block(self):
        source = '''
        ::

            >>> print('Hello, world!')
            Hello, world!
        '''
        source = stringtools.normalize(source)
        document = abjadbooktools.SphinxDocumentHandler.parse_rst(source)
        block = document[0]
        result = abjadbooktools.CodeBlock.from_docutils_literal_block(block)
        assert result == abjadbooktools.CodeBlock(
            ("print('Hello, world!')",),
            code_block_specifier=abjadbooktools.CodeBlockSpecifier(
                allow_exceptions=True,
                ),
            starting_line_number=3,
            )



# -*- coding: utf-8 -*-
from abjad.tools import sequencetools
from abjad.tools.topleveltools import new
from abjad.tools.abctools.AbjadValueObject import AbjadValueObject


class Retrograde(AbjadValueObject):
    r'''Retrograde operator.

    ..  container:: example:

        ::

            >>> Retrograde()
            Retrograde()

    Object model of twelve-tone retrograde operator.
    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_period',
        )

    ### INITIALIZER ###

    def __init__(self, period=None):
        if period is not None:
            period = abs(int(period))
            assert 0 < period
        self._period = period

    ### SPECIAL METHODS ###

    def __add__(self, operator):
        r'''Composes retrograde and `operator`.

        ..  container:: example

            Example segment:

            ::

                >>> items = [0, 2, 4, 5]
                >>> segment = PitchClassSegment(items=items)
                >>> show(segment) # doctest: +SKIP
    
            Example operators:

            ::

                >>> retrograde = Retrograde()
                >>> transposition = Transposition(n=3)

        ..  container:: example

            Transposition followed by retrograde:

            ::

                >>> operator = retrograde + transposition
                >>> str(operator)
                'RT3'

            ::

                >>> segment_ = operator(segment)
                >>> show(segment_) # doctest: +SKIP

            ..  doctest::

                >>> lilypond_file = segment_.__illustrate__()
                >>> f(lilypond_file[Voice])
                \new Voice {
                    af'8
                    g'8
                    f'8
                    ef'8
                    \bar "|."
                    \override Score.BarLine.transparent = ##f
                }

        ..  container:: example

            Same as above because retrograde and transposition commute:

            ::

                >>> operator = transposition + retrograde
                >>> str(operator)
                'T3R'

            ::

                >>> segment_ = operator(segment)
                >>> show(segment_) # doctest: +SKIP

            ..  doctest::

                >>> lilypond_file = segment_.__illustrate__()
                >>> f(lilypond_file[Voice])
                \new Voice {
                    af'8
                    g'8
                    f'8
                    ef'8
                    \bar "|."
                    \override Score.BarLine.transparent = ##f
                }

        ..  container:: example

            Returns compound operator:

            ::

                >>> f(operator)
                pitchtools.CompoundOperator(
                    operators=[
                        pitchtools.Retrograde(),
                        pitchtools.Transposition(
                            n=3,
                            ),
                        ],
                    )

        '''
        from abjad.tools import pitchtools
        return pitchtools.CompoundOperator._compose_operators(self, operator)

    def __call__(self, argument):
        r'''Calls retrograde on `argument`.

        ..  container:: example

            Gets retrograde pitch classes:

            ::

                >>> retrograde = Retrograde()
                >>> segment = PitchClassSegment([0, 1, 4, 7])
                >>> retrograde(segment)
                PitchClassSegment([7, 4, 1, 0])

        ..  container:: example

            Does not retrograde single pitches or pitch-classes:

            ::

                >>> retrogresion = Retrograde()
                >>> pitch_class = NumberedPitchClass(6)
                >>> retrograde(pitch_class)
                NumberedPitchClass(6)

        ..  container:: example

            Periodic retrograde:

            ..  todo:: Deprecated.

            ::

                >>> retrograde = Retrograde(period=3)
                >>> segment = PitchSegment("c' d' e' f' g' a' b' c''")
                >>> retrograde(segment)
                PitchSegment("e' d' c' a' g' f' c'' b'")

        Returns new object with type equal to that of `argument`.
        '''
        from abjad.tools import pitchtools
        if isinstance(argument, (pitchtools.Pitch, pitchtools.PitchClass)):
            return argument
        if not isinstance(argument, (
            pitchtools.PitchSegment,
            pitchtools.PitchClassSegment,
            )):
            argument = pitchtools.PitchSegment(argument)
        if not self.period:
            return type(argument)(reversed(argument))
        result = new(argument, items=())
        for shard in sequencetools.Sequence(argument).partition_by_counts(
            [self.period],
            cyclic=True,
            overhang=True,
            ):
            shard = type(argument)(shard)
            shard = type(argument)(reversed(shard))
            result = result + shard
        return result

    def __str__(self):
        r'''Gets string representation of operator.

        ..  container:: example

            ::

                >>> str(Retrograde())
                'R'

        '''
        return 'R'

    ### PRIVATE METHODS ###

    def _get_markup(self, direction=None):
        from abjad.tools import markuptools
        return markuptools.Markup('R', direction=direction)

    def _is_identity_operator(self):
        return False

    ### PUBLIC PROPERTIES ###

    @property
    def period(self):
        r'''Gets optional period of retrograde.

        ..  todo:: Deprecated. Use Expression followed by Retrograde instead.

        ..  container:: example

            ::

                >>> retrograde = Retrograde(period=3)
                >>> retrograde.period
                3

        Returns integer or none.
        '''
        return self._period



# -*- coding: utf-8 -*-
from __future__ import print_function
import abjad
import ide
import os
import pytest
import sys
import traceback
abjad_ide = ide.tools.idetools.AbjadIDE()


this_file = os.path.abspath(__file__)
test_directory = os.path.dirname(this_file)
inner_score_directory = os.path.dirname(test_directory)
outer_score_directory = os.path.dirname(inner_score_directory)
composer_scores_directory = os.path.dirname(outer_score_directory)
# Travis monkey patch
abjad_ide._configuration._composer_scores_directory_override = \
    composer_scores_directory
materials_directory = abjad_ide._to_score_directory(this_file, 'materials')
material_directories = abjad_ide._list_visible_paths(materials_directory)


def test_materials_01():
    r'''Interprets abbreviations file.
    '''
    abbreviations_file_path = os.path.join(
        materials_directory,
        '__abbreviations__.py',
        )
    if not os.path.exists(abbreviations_file_path):
        return
    command = 'python {{}}'.format(abbreviations_file_path)
    exit_status = abjad.systemtools.IOManager.spawn_subprocess(command)
    assert exit_status == 0


@pytest.mark.parametrize('material_directory', material_directories)
def test_materials_02(material_directory):
    r'''Checks material definition files.
    '''
    try:
        abjad_ide.check_definition_file(material_directory)
    except:
        traceback.print_exc()
        sys.exit(1)


@pytest.mark.parametrize('material_directory', material_directories)
def test_materials_03(material_directory):
    r'''Makes material PDFs.
    '''
    try:
        abjad_ide.make_pdf(material_directory)
    except:
        traceback.print_exc()
        sys.exit(1)



# -*- coding: utf-8 -*-
import subprocess
from abjad.tools.datastructuretools import TreeContainer
from abjad.tools.graphtools.GraphvizMixin import GraphvizMixin
from abjad.tools.topleveltools import new


class GraphvizGraph(GraphvizMixin, TreeContainer):
    r'''A Graphviz graph.

    ::

        >>> graph = graphtools.GraphvizGraph(name='G')

    Create other graphviz objects to insert into the graph:

    ::

        >>> cluster_0 = graphtools.GraphvizSubgraph(name='0')
        >>> cluster_1 = graphtools.GraphvizSubgraph(name='1')
        >>> a0 = graphtools.GraphvizNode(name='a0')
        >>> a1 = graphtools.GraphvizNode(name='a1')
        >>> a2 = graphtools.GraphvizNode(name='a2')
        >>> a3 = graphtools.GraphvizNode(name='a3')
        >>> b0 = graphtools.GraphvizNode(name='b0')
        >>> b1 = graphtools.GraphvizNode(name='b1')
        >>> b2 = graphtools.GraphvizNode(name='b2')
        >>> b3 = graphtools.GraphvizNode(name='b3')
        >>> start = graphtools.GraphvizNode(name='start')
        >>> end = graphtools.GraphvizNode(name='end')

    Group objects together into a tree:

    ::

        >>> graph.extend([cluster_0, cluster_1, start, end])
        >>> cluster_0.extend([a0, a1, a2, a3])
        >>> cluster_1.extend([b0, b1, b2, b3])

    Connect objects together with edges:

    ::

        >>> graphtools.GraphvizEdge().attach(start, a0)
        >>> graphtools.GraphvizEdge().attach(start, b0)
        >>> graphtools.GraphvizEdge().attach(a0, a1)
        >>> graphtools.GraphvizEdge().attach(a1, a2)
        >>> graphtools.GraphvizEdge().attach(a1, b3)
        >>> graphtools.GraphvizEdge().attach(a2, a3)
        >>> graphtools.GraphvizEdge().attach(a3, a0)
        >>> graphtools.GraphvizEdge().attach(a3, end)
        >>> graphtools.GraphvizEdge().attach(b0, b1)
        >>> graphtools.GraphvizEdge().attach(b1, b2)
        >>> graphtools.GraphvizEdge().attach(b2, b3)
        >>> graphtools.GraphvizEdge().attach(b2, a3)
        >>> graphtools.GraphvizEdge().attach(b3, end)

    Add attributes to style the objects:

    ::

        >>> cluster_0.attributes['style'] = 'filled'
        >>> cluster_0.attributes['color'] = 'lightgrey'
        >>> cluster_0.attributes['label'] = 'process #1'
        >>> cluster_0.node_attributes['style'] = 'filled'
        >>> cluster_0.node_attributes['color'] = 'white'
        >>> cluster_1.attributes['color'] = 'blue'
        >>> cluster_1.attributes['label'] = 'process #2'
        >>> cluster_1.node_attributes['style'] = ('filled', 'rounded')
        >>> start.attributes['shape'] = 'Mdiamond'
        >>> end.attributes['shape'] = 'Msquare'

    Access the computed graphviz format of the graph:

    ::

        >>> print(str(graph))
        digraph G {
            subgraph cluster_0 {
                graph [color=lightgrey,
                    label="process #1",
                    style=filled];
                node [color=white,
                    style=filled];
                a0;
                a1;
                a2;
                a3;
                a0 -> a1;
                a1 -> a2;
                a2 -> a3;
                a3 -> a0;
            }
            subgraph cluster_1 {
                graph [color=blue,
                    label="process #2"];
                node [style="filled, rounded"];
                b0;
                b1;
                b2;
                b3;
                b0 -> b1;
                b1 -> b2;
                b2 -> b3;
            }
            start [shape=Mdiamond];
            end [shape=Msquare];
            a1 -> b3;
            a3 -> end;
            b2 -> a3;
            b3 -> end;
            start -> a0;
            start -> b0;
        }

    View the graph:

    ::

        >>> topleveltools.graph(graph) # doctest: +SKIP

    Graphs can also be created without defining names.  Canonical names
    will be automatically determined for all members whose `name` is None:

    ::

        >>> graph = graphtools.GraphvizGraph()
        >>> graph.append(graphtools.GraphvizSubgraph())
        >>> graph[0].append(graphtools.GraphvizNode())
        >>> graph[0].append(graphtools.GraphvizNode())
        >>> graph[0].append(graphtools.GraphvizNode())
        >>> graph[0].append(graphtools.GraphvizSubgraph())
        >>> graph[0][-1].append(graphtools.GraphvizNode())
        >>> graph.append(graphtools.GraphvizNode())
        >>> graphtools.GraphvizEdge().attach(graph[0][1], graph[1])
        >>> graphtools.GraphvizEdge().attach(graph[0][0], graph[0][-1][0])

    ::

        >>> print(str(graph))
        digraph G {
            subgraph cluster_0 {
                node_0_0;
                node_0_1;
                node_0_2;
                subgraph cluster_0_3 {
                    node_0_3_0;
                }
                node_0_0 -> node_0_3_0;
            }
            node_1;
            node_0_1 -> node_1;
        }

    '''

    ### CLASS VARIABLES ###

    __documentation_section__ = 'Graphviz'

    __slots__ = (
        '_attributes',
        '_edge_attributes',
        '_is_digraph',
        '_node_order',
        '_node_attributes',
        )

    ### INITIALIZER ###

    def __init__(
        self,
        attributes=None,
        children=None,
        edge_attributes=None,
        is_digraph=True,
        name=None,
        node_attributes=None,
        ):
        TreeContainer.__init__(self, children=children, name=name)
        GraphvizMixin.__init__(self, attributes=attributes)
        assert isinstance(edge_attributes, (dict, type(None)))
        assert isinstance(node_attributes, (dict, type(None)))
        self._verify_attributes(edge_attributes, '_edge_attributes')
        self._verify_attributes(node_attributes, '_node_attributes')
        self._is_digraph = bool(is_digraph)
        self._node_order = None

    ### SPECIAL METHODS ###

    def __copy__(self):
        r'''Copies GraphvizGraph.

        Returns copied graph.
        '''
        copied_node, edges, mapping = self._copy_with_memo(self)
        for edge in edges:
            head, tail = edge.head, edge.tail
            if head not in mapping or tail not in mapping:
                continue
            new_edge = new(edge)
            new_edge.attach(mapping[tail], mapping[head])
        return copied_node

    def __graph__(self, **keywords):
        r'''Gets graphviz graph.

        Returns graphviz graph.
        '''
        return self

    def __str__(self):
        r'''Graphviz format of Graphviz graph.

        Returns string.
        '''
        from abjad.tools import graphtools

        edges = set([])
        for node in self.nodes[1:]:
            if isinstance(node, (
                graphtools.GraphvizField,
                graphtools.GraphvizGroup,
                )):
                continue
            if isinstance(node, graphtools.GraphvizSubgraph):
                edges.update(node._edges)
            elif isinstance(node, graphtools.GraphvizNode):
                edges.update(node.all_edges)
        edges = sorted(edges, key=lambda edge: (
            edge.tail.graph_order, edge.head.graph_order,
            ))

        edge_parents = {}
        for edge in edges:
            highest_parent = edge._get_highest_parent()
            if highest_parent not in edge_parents:
                edge_parents[highest_parent] = []
            edge_parents[highest_parent].append(edge)

        visited_edges = set()

        def recurse(node, indent=0, prefix='subgraph'):
            indent_one = indent * '    '
            indent_two = (indent + 1) * '    '
            result = []

            string = '{}{} {} {{'.format(
                indent_one,
                prefix,
                self._format_value(node.canonical_name)
                )
            result.append(string)

            if len(node.attributes):
                contributions = self._format_attribute_list(node.attributes)
                contributions[0] = 'graph {}'.format(contributions[0])
                result.extend(indent_two + x for x in contributions)
            if len(node.node_attributes):
                contributions = \
                    self._format_attribute_list(node.node_attributes)
                contributions[0] = 'node {}'.format(contributions[0])
                result.extend(indent_two + x for x in contributions)
            if len(node.edge_attributes):
                contributions = \
                    self._format_attribute_list(node.edge_attributes)
                contributions[0] = 'edge {}'.format(contributions[0])
                result.extend(indent_two + x for x in contributions)

            if indent == 0 and self._node_order:
                for node_name in self._node_order:
                    result.append('{}{};'.format(indent_two, node_name))

            for child in node:
                if isinstance(child, type(self)):
                    lines = recurse(child, indent=indent + 1)
                else:
                    lines = (indent_two + line for line in
                        child._graphviz_format_contributions)
                result.extend(lines)

            if node in edge_parents:
                edge_contributions = []
                for edge in edge_parents[node]:
                    if edge in visited_edges:
                        continue
                    for line in edge._graphviz_format_contributions:
                        edge_contributions.append(indent_two + line)
                    visited_edges.add(edge)
                result.extend(edge_contributions)

            result.append('{}}}'.format(indent_one))

            return result

        if self.is_digraph:
            return '\n'.join(recurse(self, indent=0, prefix='digraph'))
        return '\n'.join(recurse(self, indent=0, prefix='graph'))

    ### PRIVATE PROPERTIES ###

    @property
    def _node_class(self):
        from abjad.tools import graphtools
        prototype = (
            graphtools.GraphvizSubgraph,
            graphtools.GraphvizNode,
            )
        return prototype

    ### PUBLIC PROPERTIES ###

    @property
    def canonical_name(self):
        r'''Canonical name of Graphviz graph.

        Returns string.
        '''
        if self.name is not None:
            return self.name
        return 'G'

    @property
    def edge_attributes(self):
        r'''Edge attributes of Graphviz graph.
        '''
        return self._edge_attributes

    @property
    def is_digraph(self):
        r'''Is true when Graphviz graph is a digraph. Otherwise false.

        Returns true or false.
        '''
        return self._is_digraph

    @is_digraph.setter
    def is_digraph(self, arg):
        self._is_digraph = bool(arg)

    @property
    def node_attributes(self):
        r'''Node attributes of Graphviz graph.
        '''
        return self._node_attributes

    @property
    def unflattened_graphviz_format(self):
        r'''Unflattened Graphviz format of Graphviz graph.

        Returns list.
        '''
        from abjad.tools import systemtools
        assert systemtools.IOManager.find_executable(
            'unflatten'), 'Cannot find `unflatten` command-line tool.'
        graphviz_format = str(self).encode('utf-8')
        process = subprocess.Popen(
            'unflatten -l 4'.split(),
            shell=True,
            stdin=subprocess.PIPE,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
            )
        stdout, _ = process.communicate(graphviz_format)
        result = stdout.decode('utf-8')
        return result



# -*- coding: utf-8 -*-
import pytest
from abjad import *


# NOTE: all tests operate on the following expression
staff = Staff(
    r'''
    c'8
    cs'8
    <<
        \new Voice {
            d'8
            ef'8
        }
        \new Voice {
            e'8
            f'8
        }
    >>
    fs'8
    g'8
    '''
    )


def test_agenttools_IterationAgent_depth_first_01():
    '''
    Default depth-first search:
        * capped iteration returns no elements above self._client
        * unique returns each node at most once
        * no classes forbidden means all containers entered
    '''

    # LEFT-TO-RIGHT #

    iterator = iterate(staff[2]).depth_first()

    assert next(iterator) is staff[2]
    assert next(iterator) is staff[2][0]
    assert next(iterator) is staff[2][0][0]
    assert next(iterator) is staff[2][0][1]
    assert next(iterator) is staff[2][1]
    assert next(iterator) is staff[2][1][0]
    assert next(iterator) is staff[2][1][1]
    assert pytest.raises(StopIteration, 'next(iterator)')

    r'''
    Container(Voice(d'8, ef'8), Voice(e'8, f'8))
    Voice(d'8, ef'8)
    d'8
    ef'8
    Voice(e'8, f'8)
    e'8
    f'8
    '''

    # RIGHT-TO-LEFT #

    iterator = iterate(staff[2]).depth_first(
        direction=Right,
        )

    assert next(iterator) is staff[2]
    assert next(iterator) is staff[2][1]
    assert next(iterator) is staff[2][1][1]
    assert next(iterator) is staff[2][1][0]
    assert next(iterator) is staff[2][0]
    assert next(iterator) is staff[2][0][1]
    assert next(iterator) is staff[2][0][0]
    assert pytest.raises(StopIteration, 'next(iterator)')

    r'''
    Container(Voice(d'8, ef'8), Voice(e'8, f'8))
    Voice(e'8, f'8)
    f'8
    e'8
    Voice(d'8, ef'8)
    ef'8
    d'8
    '''


def test_agenttools_IterationAgent_depth_first_02():
    r'''Uncapped depth-first search: uncapped iteration returns
    all elements above self._client
    '''

    # LEFT-TO-RIGHT #

    iterator = iterate(staff[2]).depth_first(
        capped=False,
        )

    assert next(iterator) is staff[2]
    assert next(iterator) is staff[2][0]
    assert next(iterator) is staff[2][0][0]
    assert next(iterator) is staff[2][0][1]
    assert next(iterator) is staff[2][1]
    assert next(iterator) is staff[2][1][0]
    assert next(iterator) is staff[2][1][1]
    assert next(iterator) is staff
    assert next(iterator) is staff[3]
    assert next(iterator) is staff[4]
    assert pytest.raises(StopIteration, 'next(iterator)')

    r'''
    Container(Voice(d'8, ef'8), Voice(e'8, f'8))
    Voice(d'8, ef'8)
    d'8
    ef'8
    Voice(e'8, f'8)
    e'8
    f'8
    Staff{5}
    fs'8
    iterator'8
    '''

    # RIGHT-TO-LEFT #

    iterator = iterate(staff[2]).depth_first(
        capped=False,
        direction=Right,
        )

    assert next(iterator) is staff[2]
    assert next(iterator) is staff[2][1]
    assert next(iterator) is staff[2][1][1]
    assert next(iterator) is staff[2][1][0]
    assert next(iterator) is staff[2][0]
    assert next(iterator) is staff[2][0][1]
    assert next(iterator) is staff[2][0][0]
    assert next(iterator) is staff
    assert next(iterator) is staff[1]
    assert next(iterator) is staff[0]
    assert pytest.raises(StopIteration, 'next(iterator)')

    r'''
    Container(Voice(d'8, ef'8), Voice(e'8, f'8))
    Voice(e'8, f'8)
    f'8
    e'8
    Voice(d'8, ef'8)
    ef'8
    d'8
    Staff{5}
    cs'8
    c'8
    '''


def test_agenttools_IterationAgent_depth_first_03():
    r'''Duplicates-allowed depth-first search: nodes yield every time they are
    traversed.  '''

    # LEFT-TO-RIGHT #

    iterator = iterate(staff[2]).depth_first(
        unique=False,
        )

    assert next(iterator) is staff[2]
    assert next(iterator) is staff[2][0]
    assert next(iterator) is staff[2][0][0]
    assert next(iterator) is staff[2][0]
    assert next(iterator) is staff[2][0][1]
    assert next(iterator) is staff[2][0]
    assert next(iterator) is staff[2]
    assert next(iterator) is staff[2][1]
    assert next(iterator) is staff[2][1][0]
    assert next(iterator) is staff[2][1]
    assert next(iterator) is staff[2][1][1]
    assert next(iterator) is staff[2][1]
    assert next(iterator) is staff[2]
    assert pytest.raises(StopIteration, 'next(iterator)')

    r'''
    Container(Voice(d'8, ef'8), Voice(e'8, f'8))
    Voice(d'8, ef'8)
    d'8
    Voice(d'8, ef'8)
    ef'8
    Voice(d'8, ef'8)
    Container(Voice(d'8, ef'8), Voice(e'8, f'8))
    Voice(e'8, f'8)
    e'8
    Voice(e'8, f'8)
    f'8
    Voice(e'8, f'8)
    Container(Voice(d'8, ef'8), Voice(e'8, f'8))
    '''

    # RIGHT-TO-LEFT #

    iterator = iterate(staff[2]).depth_first(
        direction=Right,
        unique=False,
        )

    assert next(iterator) is staff[2]
    assert next(iterator) is staff[2][1]
    assert next(iterator) is staff[2][1][1]
    assert next(iterator) is staff[2][1]
    assert next(iterator) is staff[2][1][0]
    assert next(iterator) is staff[2][1]
    assert next(iterator) is staff[2]
    assert next(iterator) is staff[2][0]
    assert next(iterator) is staff[2][0][1]
    assert next(iterator) is staff[2][0]
    assert next(iterator) is staff[2][0][0]
    assert next(iterator) is staff[2][0]
    assert next(iterator) is staff[2]
    assert pytest.raises(StopIteration, 'next(iterator)')

    r'''
    Container(Voice(d'8, ef'8), Voice(e'8, f'8))
    Voice(e'8, f'8)
    f'8
    Voice(e'8, f'8)
    e'8
    Voice(e'8, f'8)
    Container(Voice(d'8, ef'8), Voice(e'8, f'8))
    Voice(d'8, ef'8)
    ef'8
    Voice(d'8, ef'8)
    d'8
    Voice(d'8, ef'8)
    Container(Voice(d'8, ef'8), Voice(e'8, f'8))
    '''


def test_agenttools_IterationAgent_depth_first_04():
    r'''Restricted depth-first search: iteration will yield -- but will not
    enter -- forbidden classes.
    '''

    # LEFT-TO-RIGHT #

    iterator = iterate(staff).depth_first(
        forbid='simultaneous',
        )

    assert next(iterator) is staff
    assert next(iterator) is staff[0]
    assert next(iterator) is staff[1]
    assert next(iterator) is staff[2]
    assert next(iterator) is staff[3]
    assert next(iterator) is staff[4]
    assert pytest.raises(StopIteration, 'next(iterator)')

    r'''
    Staff{5}
    c'8
    cs'8
    Container(Voice(d'8, ef'8), Voice(e'8, f'8))
    fs'8
    iterator'8
    '''

    # RIGHT-TO-LEFT #

    iterator = iterate(staff).depth_first(
        direction=Right,
        forbid='simultaneous',
        )

    assert next(iterator) is staff
    assert next(iterator) is staff[4]
    assert next(iterator) is staff[3]
    assert next(iterator) is staff[2]
    assert next(iterator) is staff[1]
    assert next(iterator) is staff[0]
    assert pytest.raises(StopIteration, 'next(iterator)')

    r'''
    Staff{5}
    iterator'8
    fs'8
    Container(Voice(d'8, ef'8), Voice(e'8, f'8))
    cs'8
    c'8
    '''


def test_agenttools_IterationAgent_depth_first_05():
    r'''Uncapped depth-first search with duplicates allowed.
    '''

    # LEFT-TO-RIGHT #

    iterator = iterate(staff[2]).depth_first(
        capped=False,
        unique=False,
        )

    assert next(iterator) is staff[2]
    assert next(iterator) is staff[2][0]
    assert next(iterator) is staff[2][0][0]
    assert next(iterator) is staff[2][0]
    assert next(iterator) is staff[2][0][1]
    assert next(iterator) is staff[2][0]
    assert next(iterator) is staff[2]
    assert next(iterator) is staff[2][1]
    assert next(iterator) is staff[2][1][0]
    assert next(iterator) is staff[2][1]
    assert next(iterator) is staff[2][1][1]
    assert next(iterator) is staff[2][1]
    assert next(iterator) is staff[2]
    assert next(iterator) is staff
    assert next(iterator) is staff[3]
    assert next(iterator) is staff
    assert next(iterator) is staff[4]
    assert next(iterator) is staff
    assert pytest.raises(StopIteration, 'next(iterator)')

    r'''
    Container(Voice(d'8, ef'8), Voice(e'8, f'8))
    Voice(d'8, ef'8)
    d'8
    Voice(d'8, ef'8)
    ef'8
    Voice(d'8, ef'8)
    Container(Voice(d'8, ef'8), Voice(e'8, f'8))
    Voice(e'8, f'8)
    e'8
    Voice(e'8, f'8)
    f'8
    Voice(e'8, f'8)
    Container(Voice(d'8, ef'8), Voice(e'8, f'8))
    Staff{5}
    fs'8
    Staff{5}
    iterator'8
    Staff{5}
    '''

    # RIGHT-TO-LEFT #

    iterator = iterate(staff[2]).depth_first(
        capped=False,
        direction=Right,
        unique=False,
        )

    assert next(iterator) is staff[2]
    assert next(iterator) is staff[2][1]
    assert next(iterator) is staff[2][1][1]
    assert next(iterator) is staff[2][1]
    assert next(iterator) is staff[2][1][0]
    assert next(iterator) is staff[2][1]
    assert next(iterator) is staff[2]
    assert next(iterator) is staff[2][0]
    assert next(iterator) is staff[2][0][1]
    assert next(iterator) is staff[2][0]
    assert next(iterator) is staff[2][0][0]
    assert next(iterator) is staff[2][0]
    assert next(iterator) is staff[2]
    assert next(iterator) is staff
    assert next(iterator) is staff[1]
    assert next(iterator) is staff
    assert next(iterator) is staff[0]
    assert next(iterator) is staff
    assert pytest.raises(StopIteration, 'next(iterator)')

    r'''
    Container(Voice(d'8, ef'8), Voice(e'8, f'8))
    Voice(e'8, f'8)
    f'8
    Voice(e'8, f'8)
    e'8
    Voice(e'8, f'8)
    Container(Voice(d'8, ef'8), Voice(e'8, f'8))
    Voice(d'8, ef'8)
    ef'8
    Voice(d'8, ef'8)
    d'8
    Voice(d'8, ef'8)
    Container(Voice(d'8, ef'8), Voice(e'8, f'8))
    Staff{5}
    cs'8
    Staff{5}
    c'8
    Staff{5}
    '''


def test_agenttools_IterationAgent_depth_first_06():
    r'''Uncapped and restricted depth-first search.
    '''

    # LEFT-TO-RIGHT #

    iterator = iterate(staff[2]).depth_first(
        capped=False,
        forbid='simultaneous',
        )

    assert next(iterator) is staff[2]
    assert next(iterator) is staff
    assert next(iterator) is staff[3]
    assert next(iterator) is staff[4]

    r'''
    Container(Voice(d'8, ef'8), Voice(e'8, f'8))
    Staff{5}
    fs'8
    iterator'8
    '''

    # RIGHT-TO-LEFT #

    iterator = iterate(staff[2]).depth_first(
        capped=False,
        direction=Right,
        forbid='simultaneous',
        )

    assert next(iterator) is staff[2]
    assert next(iterator) is staff
    assert next(iterator) is staff[1]
    assert next(iterator) is staff[0]

    r'''
    Container(Voice(d'8, ef'8), Voice(e'8, f'8))
    Staff{5}
    cs'8
    c'8
    '''


def test_agenttools_IterationAgent_depth_first_07():
    r'''Restricted depth-first search with duplicates allowed.
    '''

    # LEFT-TO-RIGHT

    iterator = iterate(staff).depth_first(
        forbid='simultaneous',
        unique=False,
        )

    assert next(iterator) is staff
    assert next(iterator) is staff[0]
    assert next(iterator) is staff
    assert next(iterator) is staff[1]
    assert next(iterator) is staff
    assert next(iterator) is staff[2]
    assert next(iterator) is staff
    assert next(iterator) is staff[3]
    assert next(iterator) is staff
    assert next(iterator) is staff[4]
    assert next(iterator) is staff
    assert pytest.raises(StopIteration, 'next(iterator)')

    r'''
    Staff{5}
    c'8
    Staff{5}
    cs'8
    Staff{5}
    Container(Voice(d'8, ef'8), Voice(e'8, f'8))
    Staff{5}
    fs'8
    Staff{5}
    iterator'8
    Staff{5}
    '''

    # RIGHT-TO-LEFT #

    iterator = iterate(staff).depth_first(
        direction=Right,
        forbid='simultaneous',
        unique=False,
        )

    assert next(iterator) is staff
    assert next(iterator) is staff[4]
    assert next(iterator) is staff
    assert next(iterator) is staff[3]
    assert next(iterator) is staff
    assert next(iterator) is staff[2]
    assert next(iterator) is staff
    assert next(iterator) is staff[1]
    assert next(iterator) is staff
    assert next(iterator) is staff[0]
    assert next(iterator) is staff
    assert pytest.raises(StopIteration, 'next(iterator)')

    r'''
    Staff{5}
    iterator'8
    Staff{5}
    fs'8
    Staff{5}
    Container(Voice(d'8, ef'8), Voice(e'8, f'8))
    Staff{5}
    cs'8
    Staff{5}
    c'8
    Staff{5}
    '''


def test_agenttools_IterationAgent_depth_first_08():
    r'''Uncapped but restricted depth-first serach with duplicates allowed.
    '''

    # LEFT-TO-RIGHT #

    iterator = iterate(staff[2]).depth_first(
        capped=False,
        forbid='simultaneous',
        unique=False,
        )

    assert next(iterator) is staff[2]
    assert next(iterator) is staff
    assert next(iterator) is staff[3]
    assert next(iterator) is staff
    assert next(iterator) is staff[4]
    assert next(iterator) is staff
    assert pytest.raises(StopIteration, 'next(iterator)')

    r'''
    Container(Voice(d'8, ef'8), Voice(e'8, f'8))
    Staff{5}
    fs'8
    Staff{5}
    iterator'8
    Staff{5}
    '''

    # RIGHT-TO-LEFT #

    iterator = iterate(staff[2]).depth_first(
        capped=False,
        direction=Right,
        forbid='simultaneous',
        unique=False,
        )

    assert next(iterator) is staff[2]
    assert next(iterator) is staff
    assert next(iterator) is staff[1]
    assert next(iterator) is staff
    assert next(iterator) is staff[0]
    assert next(iterator) is staff
    assert pytest.raises(StopIteration, 'next(iterator)')

    r'''
    Container(Voice(d'8, ef'8), Voice(e'8, f'8))
    Staff{5}
    cs'8
    Staff{5}
    c'8
    Staff{5}
    '''



# -*- coding: utf-8 -*-
import numbers
from abjad.tools.abctools.AbjadValueObject import AbjadValueObject


class StaffPosition(AbjadValueObject):
    r'''Staff position.

    ..  container:: example

        Initializes staff position at middle line of staff:

        ::

            >>> pitchtools.StaffPosition(0)
            StaffPosition(number=0)

    ..  container:: example

        Initializes staff position one space below middle line of staff:

        ::

            >>> pitchtools.StaffPosition(-1)
            StaffPosition(number=-1)

    ..  container:: example

        Initializes staff position one line below middle line of staff:

        ::

            >>> pitchtools.StaffPosition(-2)
            StaffPosition(number=-2)

    Staff positions are immutable.
    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_number',
        )

    ### INITIALIZER ###

    def __init__(self, number=0):
        assert isinstance(number, numbers.Number), repr(number)
        self._number = number

    ### SPECIAL METHODS ###

    def __eq__(self, other):
        r'''Is true when `other` is a staff position with the same number as
        this staff position. Otherwise false.

        ..  container:: example

            ::

                >>> staff_position_1 = pitchtools.StaffPosition(-2)
                >>> staff_position_2 = pitchtools.StaffPosition(-2)
                >>> staff_position_3 = pitchtools.StaffPosition(0)

            ::

                >>> staff_position_1 == staff_position_1
                True
                >>> staff_position_1 == staff_position_2
                True
                >>> staff_position_1 == staff_position_3
                False

            ::

                >>> staff_position_1 == 'foo'
                False

        Returns true or false.
        '''
        if isinstance(other, type(self)):
            return self.number == other.number
        return False

    def __float__(self):
        r'''Casts staff position as floating point number.

        ..  container:: example

            ::

                >>> float(pitchtools.StaffPosition(-2))
                -2.0

        Returns floating-point number.
        '''
        return float(self.number)

    def __hash__(self):
        r'''Hashes staff position.

        Returns integer.
        '''
        return hash(repr(self))

    def __int__(self):
        r'''Changes staff position to integer.

        ..  container:: example

            ::

                >>> int(pitchtools.StaffPosition(-2))
                -2

        Returns integer.
        '''
        return int(self.number)

    def __str__(self):
        r'''Gets string representation of staff position.

        ..  container:: example

            ::

                >>> str(pitchtools.StaffPosition(-2))
                'StaffPosition(-2)'

        Returns string.
        '''
        return '{}({})'.format(type(self).__name__, self.number)

    ### PUBLIC PROPERTIES ###

    @property
    def number(self):
        r'''Gets staff position number.

        ..  container:: example

            ::

                >>> pitchtools.StaffPosition(-2).number
                -2 

        Returns number.
        '''
        return self._number



# -*- coding: utf-8 -*-
from abjad.tools import durationtools
from abjad.tools import scoretools
from abjad.tools import systemtools
from abjad.tools.datastructuretools import TreeNode
from abjad.tools.rhythmtreetools import RhythmTreeMixin


class QGridLeaf(RhythmTreeMixin, TreeNode):
    r'''A leaf in a ``QGrid`` structure.

    ::

        >>> leaf = quantizationtools.QGridLeaf()

    ::

        >>> leaf
        QGridLeaf(
            preprolated_duration=Duration(1, 1),
            is_divisible=True
            )

    Used internally by ``QGrid``.
    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_duration',
        '_is_divisible',
        '_offset',
        '_offsets_are_current',
        '_q_event_proxies',
        )

    ### INITIALIZER ###

    def __init__(
        self,
        preprolated_duration=1,
        q_event_proxies=None,
        is_divisible=True,
        ):
        from abjad.tools import quantizationtools
        TreeNode.__init__(self)
        RhythmTreeMixin.__init__(self, preprolated_duration)
        if q_event_proxies is None:
            self._q_event_proxies = []
        else:
            assert all(isinstance(x, quantizationtools.QEventProxy)
                for x in q_event_proxies)
            self._q_event_proxies = list(q_event_proxies)
        self._is_divisible = bool(is_divisible)

    ### SPECIAL METHODS ###

    def __call__(self, pulse_duration):
        r'''Calls q-grid leaf.

        Returns selection of notes.
        '''
        pulse_duration = durationtools.Duration(pulse_duration)
        total_duration = pulse_duration * self.preprolated_duration
        return scoretools.make_notes(0, total_duration)

    def __graph__(self, **keywords):
        r'''Graphviz graph of q-grid leaf.

        Returns Graphviz graph.
        '''
        from abjad.tools import graphtools
        graph = graphtools.GraphvizGraph(name='G')
        node = graphtools.GraphvizNode(
            attributes={
                'label': str(self.preprolated_duration),
                'shape': 'box'
                }
            )
        graph.append(node)
        return graph

    ### PRIVATE METHODS ###

    def _get_format_specification(self):
        agent = systemtools.StorageFormatAgent(self)
        names = agent.signature_names
        template_names = names[:]
        if 'q_event_proxies' in names and not self.q_event_proxies:
            names.remove('q_event_proxies')
        return systemtools.FormatSpecification(
            client=self,
            repr_is_indented=True,
            storage_format_kwargs_names=names,
            template_names=template_names,
            )

    ### PRIVATE PROPERTIES ###

    @property
    def _pretty_rtm_format_pieces(self):
        return [str(self.preprolated_duration)]

    ### PUBLIC PROPERTIES ###

    @property
    def is_divisible(self):
        r'''Flag for whether the node may be further divided
        under some search tree.
        '''
        return self._is_divisible

    @is_divisible.setter
    def is_divisible(self, arg):
        self._is_divisible = bool(arg)

    @property
    def preceding_q_event_proxies(self):
        r'''Preceding q-event proxies of q-grid leaf.

        Returns list.
        '''
        return [x for x in self._q_event_proxies
            if x.offset < self.start_offset]

    @property
    def q_event_proxies(self):
        r'''Q-event proxies of q-grid leaf.
        '''
        return self._q_event_proxies

    @property
    def rtm_format(self):
        r'''RTM format of q-grid leaf.
        '''
        return str(self.preprolated_duration)

    @property
    def succeeding_q_event_proxies(self):
        r'''Succeeding q-event proxies of q-grid leaf.

        Returns list.
        '''
        return [x for x in self._q_event_proxies
            if self.start_offset <= x.offset]



# -*- coding: utf-8 -*-
from abjad.tools.scoretools.Leaf import Leaf


class MultimeasureRest(Leaf):
    '''A multimeasure rest.

    ::

        >>> rest = scoretools.MultimeasureRest((1, 4))
        >>> show(rest) # doctest: +SKIP

    '''

    ### CLASS VARIABLES ###

    __documentation_section__ = 'Leaves'

    __slots__ = (
        )

    ### INITIALIZER ###

    def __init__(self, *arguments):
        from abjad.tools import scoretools
        if len(arguments) == 0:
            arguments = ((1, 4),)
        rest = scoretools.Rest(*arguments)
        Leaf.__init__(
            self,
            rest.written_duration,
            )

    ### PRIVATE PROPERTIES ###

    @property
    def _compact_representation(self):
        return 'R%s' % self._formatted_duration

    ### PUBLIC PROPERTIES ###

    @property
    def _body(self):
        r'''List of string representation of body of rest.
        Picked up as format contribution at format-time.
        '''
        result = 'R' + str(self._formatted_duration)
        return [result]



# -*- coding: utf-8 -*-
from abjad.tools import durationtools
from abjad.tools import mathtools
from abjad.tools import stringtools
from abjad.tools.spannertools.Spanner import Spanner


class GeneralizedBeam(Spanner):
    r'''Generalized beam.

    ..  container:: example::

        ::

            >>> staff = Staff("r4 c'8 d'16 e'16 r8 fs'8 g'4")
            >>> set_(staff).auto_beaming = False
            >>> show(staff) # doctest: +SKIP

        ::

            >>> beam = spannertools.GeneralizedBeam()
            >>> attach(beam, staff[:])
            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> f(staff)
            \new Staff \with {
                autoBeaming = ##f
            } {
                r4
                c'8 [
                \set stemLeftBeamCount = 1
                \set stemRightBeamCount = 2
                d'16
                e'16 ]
                r8
                fs'8
                g'4
            }

    ..  container:: example

        ::

            >>> staff = Staff("r4 c'8 d'16 e'16 r8 fs'8 g'4")
            >>> set_(staff).auto_beaming = False
            >>> show(staff) # doctest: +SKIP

        ::

            >>> beam = spannertools.GeneralizedBeam(
            ...     isolated_nib_direction=Right,
            ...     )
            >>> attach(beam, staff[:])
            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> f(staff)
            \new Staff \with {
                autoBeaming = ##f
            } {
                r4
                c'8 [
                \set stemLeftBeamCount = 1
                \set stemRightBeamCount = 2
                d'16
                e'16 ]
                r8
                fs'8 [ ]
                g'4
            }

    ..  container:: example::

        ::

            >>> staff = Staff("r4 c'8 d'16 e'16 r8 fs'8 g'4")
            >>> set_(staff).auto_beaming = False
            >>> show(staff) # doctest: +SKIP

        ::

            >>> beam = spannertools.GeneralizedBeam(
            ...     use_stemlets=True,
            ...     )
            >>> attach(beam, staff[:])
            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> f(staff)
            \new Staff \with {
                autoBeaming = ##f
            } {
                r4
                c'8 [
                \set stemLeftBeamCount = 1
                \set stemRightBeamCount = 2
                d'16
                \set stemLeftBeamCount = 2
                \set stemRightBeamCount = 1
                e'16
                \set stemLeftBeamCount = 1
                \set stemRightBeamCount = 1
                r8
                fs'8 ]
                g'4
            }

    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_durations',
        '_include_long_duration_notes',
        '_include_long_duration_rests',
        '_isolated_nib_direction',
        '_span_points',
        '_use_stemlets',
        '_vertical_direction',
        )

    ### INITIALIZER ###

    def __init__(
        self,
        durations=None,
        include_long_duration_notes=False,
        include_long_duration_rests=False,
        isolated_nib_direction=None,
        use_stemlets=False,
        vertical_direction=None,
        ):
        Spanner.__init__(
            self,
            )
        if durations:
            durations = tuple(durationtools.Duration(x) for x in durations)
        self._durations = durations
        self._include_long_duration_notes = bool(include_long_duration_notes)
        self._include_long_duration_rests = bool(include_long_duration_rests)
        assert isolated_nib_direction in (Left, Right, None)
        self._isolated_nib_direction = isolated_nib_direction
        if self._durations is not None:
            self._span_points = mathtools.cumulative_sums(self.durations)[1:]
        else:
            self._span_points = [self._get_duration()]
        self._use_stemlets = bool(use_stemlets)
        assert vertical_direction in (Up, Down, Center, None)
        self._vertical_direction = vertical_direction

    ### PRIVATE METHODS ###

    def _get_beam_counts(
        self,
        leaf,
        previous_leaf,
        previous_leaf_is_joinable,
        next_leaf,
        next_leaf_is_joinable,
        ):

        start_offset = self._start_offset_in_me(leaf)
        stop_offset = self._stop_offset_in_me(leaf)
        left, right = None, None

        previous_flag_count = 0
        if previous_leaf is not None:
            previous_flag_count = previous_leaf.written_duration.flag_count
        next_flag_count = 0
        if next_leaf is not None:
            next_flag_count = next_leaf.written_duration.flag_count
        current_flag_count = leaf.written_duration.flag_count

        if previous_leaf_is_joinable and next_leaf_is_joinable:
            if self._is_only_leaf_in_group(start_offset, stop_offset):
                left = 1
                right = current_flag_count
            elif self._is_first_leaf_in_group(start_offset):
                left = 1
                right = current_flag_count
            elif self._is_last_leaf_in_group(stop_offset):
                left = current_flag_count
                right = 1
            else:
                left = min(previous_flag_count, current_flag_count) or 1
                right = min(current_flag_count, next_flag_count) or 1
                if left != current_flag_count and right != current_flag_count:
                    right = current_flag_count

        elif previous_leaf_is_joinable:
            if self._is_first_leaf_in_group(start_offset):
                left = 1
                right = current_flag_count
            elif self._is_my_last_leaf(leaf):
                right = None
                left = current_flag_count

        elif next_leaf_is_joinable:
            if self._is_last_leaf_in_group(stop_offset):
                right = 1
                if self._is_my_first_leaf(leaf):
                    left = current_flag_count

        return left, right

    def _get_lilypond_format_bundle(self, leaf):
        from abjad.tools import lilypondnametools
        lilypond_format_bundle = self._get_basic_lilypond_format_bundle(leaf)
        if not self._is_beamable(leaf):
            return lilypond_format_bundle
        elif not self.use_stemlets and (
            not hasattr(leaf, 'written_pitch') and
            not hasattr(leaf, 'written_pitches')):
            return lilypond_format_bundle
        leaf_ids = [id(x) for x in self._get_leaves()]
        previous_leaf = leaf._get_leaf(-1)
        previous_leaf_is_joinable = self._leaf_is_joinable(
            previous_leaf, leaf_ids)
        next_leaf = leaf._get_leaf(1)
        next_leaf_is_joinable = self._leaf_is_joinable(next_leaf, leaf_ids)
        left_beam_count, right_beam_count = self._get_beam_counts(
            leaf,
            previous_leaf,
            previous_leaf_is_joinable,
            next_leaf,
            next_leaf_is_joinable,
            )
        if left_beam_count is not None:
            context_setting = lilypondnametools.LilyPondContextSetting(
                context_property='stemLeftBeamCount',
                value=left_beam_count,
                )
            lilypond_format_bundle.update(context_setting)
        if right_beam_count is not None:
            context_setting = lilypondnametools.LilyPondContextSetting(
                context_property='stemRightBeamCount',
                value=right_beam_count,
                )
            lilypond_format_bundle.update(context_setting)
        start_piece, stop_piece = self._get_start_and_stop_pieces(
            leaf,
            previous_leaf,
            next_leaf,
            leaf_ids,
            )
        if start_piece and stop_piece:
            lilypond_format_bundle.right.spanner_starts.extend([
                start_piece, stop_piece])
        elif start_piece:
            lilypond_format_bundle.right.spanner_starts.append(start_piece)
        elif stop_piece:
            lilypond_format_bundle.right.spanner_stops.append(stop_piece)
        return lilypond_format_bundle

    def _get_start_and_stop_pieces(
        self,
        leaf,
        previous_leaf,
        next_leaf,
        leaf_ids,
        ):
        start_piece = None
        stop_piece = None
        direction_string = ''
        if self.vertical_direction is not None:
            direction_string = \
                stringtools.to_tridirectional_lilypond_symbol(
                    self.vertical_direction)
        previous_leaf_is_beamable = (
            self._is_beamable(previous_leaf) and
            id(previous_leaf) in leaf_ids
            )
        next_leaf_is_beamable = (
            self._is_beamable(next_leaf) and
            id(next_leaf) in leaf_ids
            )
        if not previous_leaf_is_beamable:
            if not next_leaf_is_beamable:
                if self.isolated_nib_direction is not None:
                    start_piece = '{}['.format(direction_string)
                    stop_piece = ']'
            else:
                start_piece = '{}['.format(direction_string)
        elif not next_leaf_is_beamable:
            stop_piece = ']'
        return start_piece, stop_piece

    def _is_beamable(self, argument, beam_rests=False):
        r'''Is true when `argument` is beamable. Otherwise false.
        '''
        from abjad.tools import scoretools
        prototype = (scoretools.Rest, scoretools.Skip)
        if beam_rests and isinstance(argument, prototype):
            return True
        if isinstance(argument, scoretools.Leaf):
            if 0 < argument.written_duration.flag_count:
                if (hasattr(argument, 'written_pitch') or
                    hasattr(argument, 'written_pitches')):
                    return True
                elif self.use_stemlets:
                    return True
        return False

    def _is_first_leaf_in_group(self, start_offset):
        if start_offset in self._span_points:
            return True
        return False

    def _is_last_leaf_in_group(self, stop_offset):
        if stop_offset in self._span_points:
            return True
        return False

    def _is_only_leaf_in_group(self, start_offset, stop_offset):
        if self._is_first_leaf_in_group(start_offset):
            if self._is_last_leaf_in_group(stop_offset):
                return True
        return False

    def _leaf_is_joinable(self, leaf, leaf_ids):
        if id(leaf) not in leaf_ids:
            return False
        if hasattr(leaf, 'written_pitch') or hasattr(leaf, 'written_pitches'):
            if self._is_beamable(leaf):
                return True
            elif self.include_long_duration_notes:
                return True
        else:
            if self._is_beamable(leaf) and self.use_stemlets:
                return True
            elif self.include_long_duration_rests:
                return True
        return False

    ### PUBLIC PROPERTIES ###

    @property
    def durations(self):
        r'''Durations to use for span-beam groupings.
        '''
        return self._durations

    @property
    def include_long_duration_notes(self):
        r'''True if beam includes long duration notes, otherwise false.
        '''
        return self._include_long_duration_notes

    @property
    def include_long_duration_rests(self):
        r'''True if beam includes long duration rests, otherwise false.
        '''
        return self._include_long_duration_rests

    @property
    def isolated_nib_direction(self):
        r'''Direction of isolated nibs.
        '''
        return self._isolated_nib_direction

    @property
    def use_stemlets(self):
        r'''True if beam uses stemlets, otherwise false.
        '''
        return self._use_stemlets

    @property
    def vertical_direction(self):
        r'''Vertical direction of the beam.
        '''
        return self._vertical_direction



# -*- coding: utf-8 -*-
from __future__ import print_function
import abc
import hashlib
import os
from abjad.tools import abctools


class ImageOutputProxy(abctools.AbjadValueObject):
    r'''Abstract base class for image output proxies.
    '''

    ### CLASS VARIABLES ###

    __documentation_section__ = 'Output Proxies'

    __slots__ = (
        '_image_layout_specifier',
        '_image_render_specifier',
        '_payload',
        )

    ### INITIALIZER ###

    def __init__(
        self,
        image_layout_specifier=None,
        image_render_specifier=None,
        ):
        self._image_layout_specifier = image_layout_specifier
        self._image_render_specifier = image_render_specifier

    ### PRIVATE METHODS ###

    def _include_graphics(
        self,
        latex_configuration,
        options,
        page_number,
        relative_file_path,
        ):
        result = []
        if page_number:
            page_number = 'page={}'.format(page_number)
            if not options:
                options = page_number
            else:
                options = options.strip()
                if not options.endswith(','):
                    options += ', '
                options += page_number
        if options:
            string = '\\noindent\\includegraphics[{}]{{{}}}'.format(
                options,
                relative_file_path,
                )
        else:
            string = '\\noindent\\includegraphics{{{}}}'.format(
                relative_file_path,
                )
        before = latex_configuration.get('before-includegraphics', ())
        after = latex_configuration.get('after-includegraphics', ())
        result.extend(before)
        result.append(string)
        result.extend(after)
        return result

    def _render_pdf_source(
        self,
        temporary_directory_path,
        ):
        raise NotImplementedError

    ### PUBLIC METHODS ###

    def as_latex(
        self,
        configuration=None,
        output_directory=None,
        relative_output_directory=None,
        ):
        r'''Creates a LaTeX representation of the image output proxy.
        '''
        import PyPDF2
        configuration = configuration or {}
        latex_configuration = configuration.get('latex', {})
        options_key = '{}-options'.format(self.file_name_prefix)
        options = latex_configuration.get(options_key, ())
        options = ''.join(options)
        file_extension = '.pdf'
        file_name = self.file_name_without_extension + file_extension
        output_directory = output_directory or relative_output_directory
        absolute_file_path = os.path.join(
            output_directory,
            file_name,
            )
        relative_file_path = os.path.join(
            relative_output_directory,
            file_name,
            )
        # Windows hack for test suite.
        relative_file_path = relative_file_path.replace(os.path.sep, '/')
        page_count = 1
        if os.path.exists(absolute_file_path):
            with open(absolute_file_path, 'rb') as file_pointer:
                pdf_reader = PyPDF2.PdfFileReader(file_pointer)
                page_count = pdf_reader.getNumPages()
        result = []
        if page_count == 1:
            result.extend(self._include_graphics(
                latex_configuration,
                options,
                None,
                relative_file_path,
                ))
        else:
            result.extend(self._include_graphics(
                latex_configuration,
                options,
                1,
                relative_file_path,
                ))
            for page_number in range(2, page_count + 1):
                result.append(r'\newline')
                result.append(r'\newline')
                result.extend(self._include_graphics(
                    latex_configuration,
                    options,
                    page_number,
                    relative_file_path,
                    ))
        return result

    def render_for_latex(
        self,
        absolute_output_directory,
        ):
        r'''Renders the image output proxy payload for LaTeX inclusion.
        '''
        target_pdf_path = os.path.join(
            absolute_output_directory,
            self.file_name_without_extension + '.pdf',
            )
        if os.path.exists(target_pdf_path):
            return
        self._render_pdf_source(absolute_output_directory)

    ### PUBLIC PROPERTIES ###

    @abc.abstractproperty
    def file_name_prefix(self):
        r'''Gets image output proxy file name prefix.

        Returns string.
        '''
        raise NotImplementedError

    @property
    def file_name_without_extension(self):
        r'''Gets image output proxy filename without file extension.

        Returns string.
        '''
        payload = '\n'.join(format(self.payload))
        md5 = hashlib.md5(payload.encode()).hexdigest()
        return '-'.join((self.file_name_prefix, md5))

    @property
    def image_layout_specifier(self):
        r'''Gets image specifier.
        '''
        return self._image_layout_specifier

    @property
    def image_render_specifier(self):
        r'''Gets image specifier.
        '''
        return self._image_render_specifier

    @property
    def payload(self):
        r'''Gets images output proxy payload.

        Returns string.
        '''
        return self._payload



# -*- coding: utf-8 -*-
from abjad.tools import stringtools
from abjad.tools.lilypondnametools.LilyPondNameManager \
    import LilyPondNameManager


class LilyPondSettingNameManager(LilyPondNameManager):
    '''LilyPond setting name manager.

    ..  container:: example

        Initializes with toplevel function:

        ::

            >>> note = Note("c'4")
            >>> set_(note)
            LilyPondSettingNameManager()

    '''

    ### SPECIAL METHODS ###

    def __getattr__(self, name):
        r'''Gets arbitrary object keyed to `name`.

        ..  container:: example

            ::

                >>> staff = Staff("c'4 d' e' f'")
                >>> set_(staff).instrument_name = Markup('Vn. I')
                >>> show(staff) # doctest: +SKIP

            ..  doctest::

                >>> f(staff)
                \new Staff \with {
                    instrumentName = \markup { "Vn. I" }
                } {
                    c'4
                    d'4
                    e'4
                    f'4
                }

        ..  container:: example

            Returns arbitrary object keyed to `name`:

            ::

                >>> set_(staff).instrument_name
                Markup(contents=['Vn. I'])

        '''
        from abjad import ly
        from abjad.tools import lilypondnametools
        camel_name = stringtools.to_upper_camel_case(name)
        if name.startswith('_'):
            try:
                return vars(self)[name]
            except KeyError:
                message = '{!r} object has no attribute: {!r}.'
                message = message.format(type(self).__name__, name)
                raise AttributeError(message)
        elif camel_name in ly.contexts:
            try:
                return vars(self)['_' + name]
            except KeyError:
                context = lilypondnametools.LilyPondNameManager()
                vars(self)['_' + name] = context
                return context
        else:
            try:
                return vars(self)[name]
            except KeyError:
                message = '{!r} object has no attribute: {!r}.'
                message = message.format(type(self).__name__, name)
                raise AttributeError(message)

    ### PRIVATE METHODS ###

    def _get_attribute_tuples(self):
        from abjad.tools import lilypondnametools
        result = []
        for name, value in vars(self).items():
            if type(value) is lilypondnametools.LilyPondNameManager:
                prefixed_context_name = name
                context_name = prefixed_context_name.strip('_')
                context_proxy = value
                attribute_pairs = context_proxy._get_attribute_pairs()
                for attribute_name, attribute_value in attribute_pairs:
                    triple = (context_name, attribute_name, attribute_value)
                    result.append(triple)
            else:
                attribute_name, attribute_value = name, value
                result.append((attribute_name, attribute_value))
        return result



# -*- coding: utf-8 -*-
import itertools
from abjad.tools import selectiontools
from abjad.tools.abctools import AbjadValueObject


class LogicalMeasureSelectorCallback(AbjadValueObject):
    r'''Logical measure selector callback.

    ..  container:: example

        Score for examples 1 - 3:

        ::

            >>> staff = Staff("c'8 d' e' f' g' a' b' c''")
            >>> attach(TimeSignature((2, 8)), staff[0])
            >>> attach(TimeSignature((3, 8)), staff[4])
            >>> attach(TimeSignature((1, 8)), staff[7])
            >>> show(staff) # doctest: +SKIP

    ..  container:: example

        **Example 1.** Selects the leaves of every logical measure:

        ::

            >>> selector = selectortools.Selector()
            >>> selector = selector.by_leaf()
            >>> selector = selector.by_logical_measure()
            >>> for x in selector(staff):
            ...     x
            ...
            Selection([Note("c'8"), Note("d'8")])
            Selection([Note("e'8"), Note("f'8")])
            Selection([Note("g'8"), Note("a'8"), Note("b'8")])
            Selection([Note("c''8")])

    ..  container:: example

        **Example 2.** Selects the first leaf of every logical measure:

        ::

            >>> selector = selectortools.Selector()
            >>> selector = selector.by_leaf()
            >>> selector = selector.by_logical_measure()
            >>> selector = selector[0]
            >>> selector(staff)
            Selection([Note("c'8"), Note("e'8"), Note("g'8"), Note("c''8")])

    ..  container:: example

        **Example 3.** Selects the last leaf of every logical measure:

        ::

            >>> selector = selectortools.Selector()
            >>> selector = selector.by_leaf()
            >>> selector = selector.by_logical_measure()
            >>> selector = selector[-1]
            >>> selector(staff)
            Selection([Note("d'8"), Note("f'8"), Note("b'8"), Note("c''8")])

    ..  container:: example

        **Example 4.** Works with implicit time signatures:

        ::

            >>> staff = Staff("c'4 d' e' f' g' a' b' c''")
            >>> score = Score([staff])
            >>> scheme = schemetools.SchemeMoment((1, 16))
            >>> set_(score).proportional_notation_duration = scheme
            >>> show(score) # doctest: +SKIP

        ::

            >>> selector = selectortools.Selector()
            >>> selector = selector.by_leaf()
            >>> selector = selector.by_logical_measure()
            >>> for x in selector(score):
            ...     x
            ...
            Selection([Note("c'4"), Note("d'4"), Note("e'4"), Note("f'4")])
            Selection([Note("g'4"), Note("a'4"), Note("b'4"), Note("c''4")])

    Groups components by the logical measure of component start offset.
    '''

    ### CLASS VARIABLES ###

    __documentation_section__ = 'Callbacks'

    __slots__ = (
        )

    ### INITIALIZER ###

    def __init__(self):
        pass

    ### SPECIAL METHODS ###

    def __call__(self, argument, rotation=None):
        r'''Iterates tuple `argument`.

        Returns tuple of selections.
        '''
        assert isinstance(argument, tuple), repr(argument)
        selections = []
        for subexpr in argument:
            selections_ = self._group(subexpr)
            selections.extend(selections_)
        return tuple(selections)

    ### PRIVATE METHODS ###


    def _get_first_component(self, argument):
        from abjad.tools import scoretools
        if isinstance(argument, scoretools.Component):
            return argument
        else:
            component = argument[0]
            assert isinstance(component, scoretools.Component)
            return component

    def _get_logical_measure_number(self, argument):
        first_component = self._get_first_component(argument)
        assert first_component._logical_measure_number is not None
        return first_component._logical_measure_number

    def _group(self, argument):
        selections = []
        first_component = self._get_first_component(argument)
        first_component._update_logical_measure_numbers()
        pairs = itertools.groupby(
            argument,
            lambda _: self._get_logical_measure_number(_),
            )
        for value, group in pairs:
            selection = selectiontools.Selection(list(group))
            selections.append(selection)
        return tuple(selections)



# -*- coding: utf-8 -*-
import pytest
from abjad import *


def test_indicatortools_TimeSignature_suppress_01():
    r'''Suppress time signature with power-of-two denominator at format-time.
    '''

    measure = Measure((7, 8), "c'8 d'8 e'8 f'8 g'8 a'8 b'8")
    measure.time_signature.suppress = True

    assert format(measure) == stringtools.normalize(
        r'''
        {
            c'8
            d'8
            e'8
            f'8
            g'8
            a'8
            b'8
        }
        '''
        )


def test_indicatortools_TimeSignature_suppress_02():
    r'''Suppressing time signature without power-of-two denominator
    raises exception.
    '''

    measure = Measure((8, 9), "c'8 d'8 e'8 f'8 g'8 a'8 b'8 c''8")
    measure.time_signature.suppress = True

    assert pytest.raises(Exception, 'format(measure)')



# -*- coding: utf-8 -*-
from abjad.tools.documentationtools.ReSTDirective import ReSTDirective


class ReSTLineageDirective(ReSTDirective):
    r'''A ReST lineage directive.

    Digrams inheritance of Abjad classes.

    ::

        >>> documentationtools.ReSTLineageDirective(argument=spannertools.Beam)
        ReSTLineageDirective(
            argument='abjad.tools.spannertools.Beam.Beam'
            )

    ::

        >>> print(_.rest_format)
        .. abjad-lineage:: abjad.tools.spannertools.Beam.Beam

    '''

    ### CLASS VARIABLES ###

    __documentation_section__ = 'reStructuredText'

    ### INITIALIZER ###

    def __init__(self, argument=None, children=None, name=None, options=None):
        if isinstance(argument, type):
            argument = argument.__module__ + '.' + argument.__name__
        ReSTDirective.__init__(
            self,
            argument=argument,
            children=children,
            name=name,
            options=options,
            )

    ### PUBLIC PROPERTIES ###

    @property
    def directive(self):
        r'''Returns ``'abjad-lineage'``.
        '''
        return 'abjad-lineage'



# -*- coding: utf-8 -*-
from abjad import *


def test_scoretools_make_multimeasure_rests_01():

    mmrs = scoretools.make_multimeasure_rests([(4, 8), (6, 8), (7, 8)])
    staff = Staff(mmrs)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            R2
            R2.
            R2..
        }
        '''
        )

    format(staff) == '\\new Staff {\n\tR2\n\tR2.\n\tR2..\n}'



# -*- coding: utf-8 -*-
import copy
import functools
from abjad.tools import mathtools
from abjad.tools.pitchtools.Interval import Interval
from abjad.tools.topleveltools import new


@functools.total_ordering
class NamedInterval(Interval):
    '''Named interval.

    ..  container:: example

        Initializes ascending major ninth from string:

        ::

            >>> interval = NamedInterval('+M9')
            >>> interval
            NamedInterval('+M9')

    ..  container:: example

        Initializes descending major third from number of semitones:

        ::

            >>> interval = NamedInterval(-4)
            >>> interval
            NamedInterval('-M3')

    '''

    ### CLASS VARIABLES ##

    __slots__ = (
        '_number',
        '_quality_string',
        )

    _acceptable_quality_strings = (
        'perfect',
        'major',
        'minor',
        'diminished',
        'augmented',
        )

    _quality_abbreviation_to_quality_string = {
        'M': 'major',
        'm': 'minor',
        'P': 'perfect',
        'aug': 'augmented',
        'dim': 'diminished',
        }

    _semitones_to_quality_string_and_number = {
        0: ('perfect', 1),
        1: ('minor', 2),
        2: ('major', 2),
        3: ('minor', 3),
        4: ('major', 3),
        5: ('perfect', 4),
        6: ('diminished', 5),
        7: ('perfect', 5),
        8: ('minor', 6),
        9: ('major', 6),
        10: ('minor', 7),
        11: ('major', 7),
        }

    ### INITIALIZER ###

    def __init__(self, *arguments):
        from abjad.tools import pitchtools
        if len(arguments) == 1:
            if isinstance(arguments[0], type(self)):
                quality_string = arguments[0].quality_string
                number = arguments[0].number
            elif isinstance(arguments[0], str):
                match = \
                    pitchtools.Interval._interval_name_abbreviation_regex.match(
                        arguments[0])
                if match is None:
                    message = '{!r} does not have the form of a mdi abbreviation.'
                    message = message.format(arguments[0])
                    raise ValueError(message)
                direction_string, quality_abbreviation, number_string = \
                    match.groups()
                quality_string = self._quality_abbreviation_to_quality_string[
                    quality_abbreviation]
                number = int(direction_string + number_string)
            elif isinstance(arguments[0], pitchtools.NamedIntervalClass):
                quality_string = arguments[0].quality_string
                number = arguments[0].number
            elif isinstance(arguments[0], (
                int,
                float,
                int,
                pitchtools.NumberedInterval,
                pitchtools.NumberedIntervalClass,
                )):
                number = int(arguments[0])
                sign = mathtools.sign(number)
                octaves, semitones = divmod(abs(number), 12)
                quality_string, number = \
                    self._semitones_to_quality_string_and_number[semitones]
                number += abs(octaves) * 7
                if sign == -1:
                    number *= -1
            else:
                message = 'can not initialize {}: {!r}'
                message = message.format(type(self).__init__, arguments)
                raise ValueError(message)
        elif len(arguments) == 2:
            quality_string, number = arguments
        elif len(arguments) == 0:
            quality_string = 'perfect'
            number = 1
        else:
            message = 'can not initialize {}: {!r}'
            message = message.format(type(self).__init__, arguments)
            raise ValueError(message)
        self._quality_string = quality_string
        self._number = number

    ### SPECIAL METHODS ###

    def __abs__(self):
        r'''Absolute value of named interval.

        ::

            >>> interval = NamedInterval('+M9')
            >>> abs(interval)
            NamedInterval('+M9')

        Returns named interval.
        '''
        return type(self)(self.quality_string, abs(self.number))

    def __add__(self, argument):
        r'''Adds `argument` to named interval.

        ::

            >>> interval + NamedInterval('M2')
            NamedInterval('+M10')

        Returns new named interval.
        '''
        from abjad.tools import pitchtools
        if not isinstance(argument, type(self)):
            message = 'must be named interval: {!r}.'
            message = message.format(argument)
            raise TypeError(message)
        dummy_pitch = pitchtools.NamedPitch(0)
        new_pitch = dummy_pitch + self + argument
        return pitchtools.NamedInterval.from_pitch_carriers(
            dummy_pitch, new_pitch)

    def __copy__(self, *arguments):
        r'''Copies named interval.

        ::

            >>> import copy
            >>> copy.copy(interval)
            NamedInterval('+M9')

        Returns new named interval.
        '''
        return type(self)(self.quality_string, self.number)

    def __float__(self):
        r'''Changes number of named interval to a float.

        ::

            >>> float(interval)
            9.0

        Returns float.
        '''
        return float(self._number)

    def __int__(self):
        r'''Returns number of named interval.

        ::

            >>> int(interval)
            9

        Returns integer.
        '''
        return self._number

    def __lt__(self, argument):
        r'''Is true when `argument` is a named interval with a number greater than that
        of this named interval.

        ::

            >>> interval < NamedInterval('+M10')
            True

        Also true when `argument` is a named interval with a
        number equal to this named interval and with semitones greater than
        this named interval:

        ::

            >>> NamedInterval('+m9') < interval
            True

        Otherwise false:

        ::

            >>> interval < NamedInterval('+M2')
            False

        Returns true or false.
        '''
        if isinstance(argument, type(self)):
            if self.number == argument.number:
                return self.semitones < argument.semitones
            return self.number < argument.number
        return False

    def __mul__(self, argument):
        r'''Multiplies named interval by `argument`.

        ::

            >>> 3 * interval
            NamedInterval('+aug25')

        Returns new named interval.
        '''
        from abjad.tools import pitchtools
        if not isinstance(argument, int):
            message = 'must be integer: {!r}.'
            message = message.format(argument)
            raise TypeError(message)
        dummy_pitch = pitchtools.NamedPitch(0)
        for i in range(abs(argument)):
            dummy_pitch += self
        result = pitchtools.NamedInterval.from_pitch_carriers(
            pitchtools.NamedPitch(0), dummy_pitch)
        if argument < 0:
            return -result
        return result

    def __ne__(self, argument):
        r'''Is true when `argument` does not equal this named interval. Otherwise
        false.

        Returns true or false.
        '''
        return not self == argument

    def __neg__(self):
        r'''Negates named interval.

        ::

            >>> -interval
            NamedInterval('-M9')

        Returns new named interval.
        '''
        return type(self)(self.quality_string, -self.number)

    def __rmul__(self, argument):
        r'''Multiplies `argument` by named interval.

        ::

            >>> interval * 3
            NamedInterval('+aug25')

        Returns new named interval.
        '''
        return self * argument

    def __str__(self):
        r'''String representation of named interval.

        ::

            >>> str(interval)
            '+M9'

        Returns string.
        '''
        return '{}{}{}'.format(
            self._direction_symbol,
            self._quality_abbreviation,
            abs(self.number),
            )

    def __sub__(self, argument):
        r'''Subtracts `argument` from named interval.

        ::

            >>> interval - NamedInterval('+M2')
            NamedInterval('+P8')

        Returns new named interval.
        '''
        from abjad.tools import pitchtools
        if not isinstance(argument, type(self)):
            message = 'must be named interval: {!r}.'
            message = message.format(argument)
            raise TypeError(message)
        dummy_pitch = pitchtools.NamedPitch(0)
        new_pitch = dummy_pitch + self - argument
        return pitchtools.NamedInterval.from_pitch_carriers(
            dummy_pitch, new_pitch)

    ### PRIVATE PROPERTIES ###

    @property
    def _format_string(self):
        return '{}{}'.format(self._quality_abbreviation, self.number)

    @property
    def _interval_string(self):
        interval_to_string = {
            1: 'unison',
            2: 'second',
            3: 'third',
            4: 'fourth',
            5: 'fifth',
            6: 'sixth',
            7: 'seventh',
            8: 'octave',
            9: 'ninth',
            10: 'tenth',
            11: 'eleventh',
            12: 'twelth',
            13: 'thirteenth',
            14: 'fourteenth',
            15: 'fifteenth',
            }
        try:
            interval_string = interval_to_string[abs(self.number)]
        except KeyError:
            abs_number = abs(self.number)
            residue = abs_number % 10
            if residue == 1:
                suffix = 'st'
            elif residue == 2:
                suffix = 'nd'
            elif residue == 3:
                suffix = 'rd'
            else:
                suffix = 'th'
            interval_string = '%s%s' % (abs_number, suffix)
        return interval_string

    @property
    def _quality_abbreviation(self):
        _quality_string_to_quality_abbreviation = {
            'major': 'M', 'minor': 'm', 'perfect': 'P',
            'augmented': 'aug', 'diminished': 'dim'}
        return _quality_string_to_quality_abbreviation[self.quality_string]

    ### PRIVATE METHODS ###

    def _get_format_specification(self):
        superclass = super(NamedInterval, self)
        format_specification = superclass._get_format_specification()
        return new(
            format_specification,
            template_names=['quality_string', 'number'],
            )

    def _transpose_pitch(self, pitch):
        from abjad.tools import pitchtools
        pitch_number = pitch.pitch_number + self.semitones
        diatonic_pitch_class_number = \
            (pitch.diatonic_pitch_class_number + self.staff_spaces) % 7
        diatonic_pitch_class_name = \
            pitchtools.PitchClass._diatonic_pitch_class_number_to_diatonic_pitch_class_name[
                diatonic_pitch_class_number]
        named_pitch = pitchtools.NamedPitch(
            pitch_number, diatonic_pitch_class_name)
        return type(pitch)(named_pitch)

    ### PUBLIC PROPERTIES ###

    @property
    def direction_number(self):
        r'''Direction number of named interval.

        ::

            >>> interval.direction_number
            1

        Returns ``-1``, ``0`` or ``1``.
        '''
        if self.quality_string == 'perfect' and abs(self.number) == 1:
            return 0
        else:
            return mathtools.sign(self.number)

    @property
    def direction_string(self):
        r'''Direction string of named interval.

        ::

            >>> interval.direction_string
            'ascending'

        Returns ``'ascending'``, ``'descending'`` or none.
        '''
        if self.direction_number == -1:
            return 'descending'
        elif self.direction_number == 0:
            return None
        elif self.direction_number == 1:
            return 'ascending'

    @property
    def interval_class(self):
        r'''Interval class of named interval.

        ::

            >>> interval.interval_class
            2

        Returns nonnegative integer.
        '''
        return ((abs(self.number) - 1) % 7) + 1

    @property
    def interval_string(self):
        r'''Interval string of named interval.

        ::

            >>> interval.interval_string
            'ninth'

        Returns string.
        '''
        return self._interval_string

    @property
    def named_interval_class(self):
        r'''Named interval class of named interval.

        ::

            >>> interval.named_interval_class
            NamedIntervalClass('+M2')

        Returns named inversion-equivalent interval-class.
        '''
        from abjad.tools import pitchtools
        quality_string, number = self._quality_string, self.number
        return pitchtools.NamedIntervalClass(quality_string, number)

    @property
    def number(self):
        r'''Number of named interval.

        ::

            >>> interval.number
            9

        Returns nonnegative number.
        '''
        return self._number

    @property
    def octaves(self):
        r'''Number of octaves in interval.

        Returns nonnegative number.
        '''
        return self.semitones // 12

    @property
    def quality_string(self):
        r'''Quality string of named interval.

        ::

            >>> interval.quality_string
            'major'

        Returns string.
        '''
        return self._quality_string

    @property
    def semitones(self):
        r'''Semitones of named interval.

        ::

            >>> interval.semitones
            14

        Returns number.
        '''
        from abjad.tools import pitchtools
        result = 0
        interval_class_number_to_semitones = {
            1: 0,
            2: 1,
            3: 3,
            4: 5,
            5: 7,
            6: 8,
            7: 10,
            8: 0,
            }
        perfect_interval_classes = (
            1,
            4,
            5,
            8,
            )
        interval_class_number = abs(
            pitchtools.NamedIntervalClass(self).number)
        result += interval_class_number_to_semitones[interval_class_number]
        result += (abs(self.number) - 1) // 7 * 12
        quality_string_to_semitones = {
            'perfect': 0,
            'major': 1,
            'minor': 0,
            'augmented': 1,
            'diminished': -1,
            }
        result += quality_string_to_semitones[self.quality_string]
        if interval_class_number not in perfect_interval_classes and \
            self.quality_string == "augmented":
            result += 1
        if self.number < 0:
            result *= -1
        return result

    @property
    def staff_spaces(self):
        r'''Staff spaces of named interval.

        ::

            >>> interval.staff_spaces
            8

        Returns nonnegative integer.
        '''
        if self.direction_string == 'descending':
            return self.number + 1
        elif self.direction_string is None:
            return 0
        elif self.direction_string == 'ascending':
            return self.number - 1

    ### PUBLIC METHODS ###

    @classmethod
    def from_pitch_carriers(class_, pitch_carrier_1, pitch_carrier_2):
        '''Calculate named interval from `pitch_carrier_1` to
        `pitch_carrier_2`:

        ::

            >>> NamedInterval.from_pitch_carriers(
            ...     NamedPitch(-2),
            ...     NamedPitch(12),
            ...     )
            NamedInterval('+M9')

        ::

            ..  todo:: Improve this behavior.

            >>> NamedInterval.from_pitch_carriers(
            ...     NamedPitch("cs'"),
            ...     NamedPitch("cf'"),
            ...     )
            NamedInterval('-M2')

        Returns named interval.
        '''
        from abjad.tools import pitchtools
        pitch_1 = pitchtools.NamedPitch.from_pitch_carrier(pitch_carrier_1)
        pitch_2 = pitchtools.NamedPitch.from_pitch_carrier(pitch_carrier_2)
        degree_1 = pitch_1.diatonic_pitch_number
        degree_2 = pitch_2.diatonic_pitch_number
        named_interval_number = abs(degree_1 - degree_2) + 1
        numbered_interval_number = abs(
            pitchtools.NumberedPitch(pitch_1).pitch_number -
            pitchtools.NumberedPitch(pitch_2).pitch_number
            )
        numbered_interval = pitchtools.NumberedInterval(
            numbered_interval_number,
            )
        absolute_named_interval = numbered_interval.to_named_interval(
            named_interval_number
            )
        if pitch_2 < pitch_1:
            named_interval = -absolute_named_interval
        else:
            named_interval = absolute_named_interval
        return class_(named_interval)

    def transpose(self, pitch_carrier):
        r'''Transposes `pitch_carrier`.

        ..  container:: example

            Transposes chord:

            ::

                >>> chord = Chord("<c' e' g'>4")

            ::

                >>> interval = NamedInterval('+m2')
                >>> interval.transpose(chord)
                Chord("<df' f' af'>4")

        Returns new (copied) object of `pitch_carrier` type.
        '''
        from abjad.tools import pitchtools
        from abjad.tools import scoretools
        if isinstance(pitch_carrier, pitchtools.Pitch):
            return self._transpose_pitch(pitch_carrier)
        elif isinstance(pitch_carrier, scoretools.Note):
            new_note = copy.copy(pitch_carrier)
            new_pitch = self._transpose_pitch(pitch_carrier.written_pitch)
            new_note.written_pitch = new_pitch
            return new_note
        elif isinstance(pitch_carrier, scoretools.Chord):
            new_chord = copy.copy(pitch_carrier)
            pairs = zip(new_chord.note_heads, pitch_carrier.note_heads)
            for new_nh, old_nh in pairs:
                new_pitch = self._transpose_pitch(old_nh.written_pitch)
                new_nh.written_pitch = new_pitch
            return new_chord
        else:
            return pitch_carrier



# -*- coding: utf-8 -*-
import numbers
from abjad.tools.pitchtools.NamedIntervalClass import NamedIntervalClass


class NamedInversionEquivalentIntervalClass(NamedIntervalClass):
    '''Named inversion-equivalent interval-class.

    ..  container:: example

        Initializes from string:

        ::

            >>> pitchtools.NamedInversionEquivalentIntervalClass('-m14')
            NamedInversionEquivalentIntervalClass('+M2')

    Named inversion-equivalent interval-classes are immutable.
    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_number',
        '_quality_string',
        )

    ### INITIALIZER ###

    def __init__(self, *arguments):
        from abjad.tools import pitchtools
        if len(arguments) == 1 and isinstance(arguments[0], type(self)):
            self._initialize_by_self_reference(arguments[0])
        elif len(arguments) == 1 and isinstance(arguments[0], str):
            self._initialize_by_string(arguments[0])
        elif len(arguments) == 1 and isinstance(arguments[0],
            pitchtools.NamedIntervalClass):
            self._initialize_by_string(str(arguments[0]))
        elif len(arguments) == 1 and isinstance(arguments[0],
            pitchtools.NamedInterval):
            interval_class = pitchtools.NamedIntervalClass(arguments[0])
            self._initialize_by_string(str(interval_class))
        elif len(arguments) == 1 and isinstance(arguments[0], tuple):
            self._initialize_by_quality_string_and_number(*arguments[0])
        elif len(arguments) == 2:
            self._initialize_by_quality_string_and_number(*arguments)
        elif len(arguments) == 0:
            self._initialize_by_string('P1')
        else:
            message = 'can not initialize {}: {!r}.'
            message = message.format(type(self).__name__, arguments)
            raise ValueError(message)

    ### SPECIAL METHODS ###

    def __eq__(self, argument):
        r'''Is true when `argument` is a named inversion-equivalent interval-class with
        quality string and number equal to those of this named
        inversion-equivalent interval-class. Otherwise false.

        Returns true or false.
        '''
        if isinstance(argument, type(self)):
            if self.quality_string == argument.quality_string:
                if self.number == argument.number:
                    return True
        return False

    def __hash__(self):
        r'''Required to be explicitly redefined on Python 3 if
        __eq__ changes

        Returns integer.
        '''
        return super(NamedInversionEquivalentIntervalClass, self).__hash__()

    def __ne__(self, argument):
        r'''Is true when named inversion-equivalent interval-class does not equal
        `argument`. Otherwise false.

        Returns true or false.
        '''
        return not self == argument

    ### PRIVATE METHODS ###

    def _initialize_by_quality_string_and_number(self, quality_string, number):
        if number == 0:
            message = 'named interval can not equal zero.'
            raise ValueError(message)
        elif abs(number) == 1:
            number = 1
        elif abs(number) % 7 == 0:
            number = 7
        elif abs(number) % 7 == 1:
            number = 8
        else:
            number = abs(number) % 7
        if self._is_representative_number(number):
            quality_string = quality_string
            number = number
        else:
            quality_string = self._invert_quality_string(quality_string)
            number = 9 - number
        self._quality_string = quality_string
        self._number = number

    def _initialize_by_self_reference(self, reference):
        quality_string = reference.quality_string
        number = reference.number
        self._initialize_by_quality_string_and_number(quality_string, number)

    def _initialize_by_string(self, string):
        from abjad.tools import pitchtools
        match = pitchtools.Interval._interval_name_abbreviation_regex.match(string)
        if match is None:
            raise ValueError(
                '{!r} does not have the form of a hdi abbreviation.'.format(
                string))
        direction_string, quality_abbreviation, number_string = \
            match.groups()
        quality_string = self._quality_abbreviation_to_quality_string[
            quality_abbreviation]
        number = int(number_string)
        self._initialize_by_quality_string_and_number(quality_string, number)

    def _invert_quality_string(self, quality_string):
        inversions = {
            'major': 'minor',
            'minor': 'major',
            'perfect': 'perfect',
            'augmented': 'diminished',
            'diminished': 'augmented',
            }
        return inversions[quality_string]

    def _is_representative_number(self, argument):
        if isinstance(argument, numbers.Number):
            if 1 <= argument <= 4 or argument == 8:
                return True
        return False

    ### PUBLIC METHODS ###

    @classmethod
    def from_pitch_carriers(class_, pitch_carrier_1, pitch_carrier_2):
        '''Makes named inversion-equivalent interval-class from
        `pitch_carrier_1` and `pitch_carrier_2`.

        ::

            >>> pitchtools.NamedInversionEquivalentIntervalClass.from_pitch_carriers(
            ...     NamedPitch(-2),
            ...     NamedPitch(12),
            ...     )
            NamedInversionEquivalentIntervalClass('+M2')

        Returns named inversion-equivalent interval-class.
        '''
        from abjad.tools import pitchtools
        named_interval = pitchtools.NamedInterval.from_pitch_carriers(
            pitch_carrier_1, pitch_carrier_2)
        return class_(named_interval)
